@@ -13,7 +13,7 @@
  *  * Neither the name of the ALICE Project-Team nor the names of its
  *  contributors may be used to endorse or promote products derived from this
  *  software without specific prior written permission.
- * 
+ *
  *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -45,9 +45,9 @@
 #include "nl_blas.h"
 
 /*
- Some warnings about const cast in callback for
- qsort() function.
- */
+  Some warnings about const cast in callback for
+  qsort() function.
+*/
 
 #ifdef __clang__
 #pragma GCC diagnostic ignored "-Wcast-qual"
@@ -174,7 +174,7 @@ NLboolean nlCRSMatrixSave(NLCRSMatrix* M, const char* filename) {
     fwrite(M->rowptr, sizeof(NLuint_big), M->m+1, f);
     fwrite(M->colind, sizeof(NLuint), nnz, f);
     fwrite(M->val, sizeof(double), nnz, f);
-    
+
     return NL_TRUE;
 }
 
@@ -182,12 +182,12 @@ NLboolean nlCRSMatrixLoad(NLCRSMatrix* M, const char* filename) {
     NLuint_big nnz = 0;
     FILE* f = fopen(filename, "rb");
     NLboolean truncated = NL_FALSE;
-    
+
     if(f == NULL) {
         nlError("nlCRSMatrixLoad", "Could not open file");
         return NL_FALSE;
     }
-    
+
     truncated = truncated || (
         fread(&M->m, sizeof(NLuint), 1, f) != 1 ||
         fread(&M->n, sizeof(NLuint), 1, f) != 1 ||
@@ -216,7 +216,7 @@ NLboolean nlCRSMatrixLoad(NLCRSMatrix* M, const char* filename) {
         NL_DELETE_ARRAY(M->val);
         return NL_FALSE;
     } else {
-        M->nslices = 1;    
+        M->nslices = 1;
         M->sliceptr = NL_NEW_ARRAY(NLuint, M->nslices+1);
         M->sliceptr[0] = 0;
         M->sliceptr[1] = M->m;
@@ -240,7 +240,7 @@ static void nlCRSMatrixMultSlice(
         for(j=M->rowptr[i]; j<M->rowptr[i+1]; ++j) {
             sum += M->val[j] * x[M->colind[j]];
         }
-        y[i] = sum; 
+        y[i] = sum;
     }
 }
 
@@ -259,7 +259,7 @@ static void nlCRSMatrixMult(
     NLuint i,j;
     NLuint_big jj;
     NLdouble a;
-    
+
     if(M->symmetric_storage) {
         for(i=0; i<M->m; ++i) {
             y[i] = 0.0;
@@ -275,16 +275,16 @@ static void nlCRSMatrixMult(
             }
         }
     } else {
-    
+
 #if defined(_OPENMP)
 #pragma omp parallel for private(slice)
 #endif
-    
-	for(slice=0; slice<nslices; ++slice) {
-	    nlCRSMatrixMultSlice(
-		M,x,y,M->sliceptr[slice],M->sliceptr[slice+1]
-	    );
-	}
+
+        for(slice=0; slice<nslices; ++slice) {
+            nlCRSMatrixMultSlice(
+                M,x,y,M->sliceptr[slice],M->sliceptr[slice+1]
+            );
+        }
     }
 
     nlHostBlas()->flops += (NLulong)(2*nlCRSMatrixNNZ(M));
@@ -298,9 +298,9 @@ void nlCRSMatrixConstruct(
     M->type = NL_MATRIX_CRS;
     M->destroy_func = (NLDestroyMatrixFunc)nlCRSMatrixDestroy;
     if(NLMultMatrixVector_MKL != NULL) {
-	M->mult_func = (NLMultMatrixVectorFunc)NLMultMatrixVector_MKL;
+        M->mult_func = (NLMultMatrixVectorFunc)NLMultMatrixVector_MKL;
     } else {
-	M->mult_func = (NLMultMatrixVectorFunc)nlCRSMatrixMult;
+        M->mult_func = (NLMultMatrixVectorFunc)nlCRSMatrixMult;
     }
     M->nslices = nslices;
     M->val = NL_NEW_ARRAY(double, nnz);
@@ -335,9 +335,9 @@ void nlCRSMatrixConstructPattern(
     M->type = NL_MATRIX_CRS;
     M->destroy_func = (NLDestroyMatrixFunc)nlCRSMatrixDestroy;
     if(NLMultMatrixVector_MKL != NULL) {
-	M->mult_func = (NLMultMatrixVectorFunc)NLMultMatrixVector_MKL;
+        M->mult_func = (NLMultMatrixVectorFunc)NLMultMatrixVector_MKL;
     } else {
-	M->mult_func = (NLMultMatrixVectorFunc)nlCRSMatrixMult;
+        M->mult_func = (NLMultMatrixVectorFunc)nlCRSMatrixMult;
     }
     M->nslices = 0;
     M->val = NULL;
@@ -372,7 +372,7 @@ void nlCRSMatrixPatternSetRowLength(
     nl_assert(M->colind == NULL);
     nl_assert(M->val == NULL);
     /* Store row length in rowptr */
-    M->rowptr[i+1] = (NLuint_big)(n); 
+    M->rowptr[i+1] = (NLuint_big)(n);
 }
 
 void nlCRSMatrixComputeSlices(NLCRSMatrix* CRS);
@@ -383,19 +383,19 @@ void nlCRSMatrixComputeSlices(NLCRSMatrix* CRS) {
     NLuint_big cur_bound, cur_NNZ;
     /* Create "slices" to be used by parallel sparse matrix vector product */
     if(CRS->sliceptr != NULL) {
-	cur_bound = slice_size;
-	cur_NNZ = 0;
-	cur_row = 0;
-	CRS->sliceptr[0]=0;
-	for(slice=1; slice<CRS->nslices; ++slice) {
-	    while(cur_NNZ < cur_bound && cur_row < CRS->m) {
-		++cur_row;
-		cur_NNZ += CRS->rowptr[cur_row+1] - CRS->rowptr[cur_row];
-	    }
-	    CRS->sliceptr[slice] = cur_row;
-	    cur_bound += slice_size;
-	}
-	CRS->sliceptr[CRS->nslices]=CRS->m;
+        cur_bound = slice_size;
+        cur_NNZ = 0;
+        cur_row = 0;
+        CRS->sliceptr[0]=0;
+        for(slice=1; slice<CRS->nslices; ++slice) {
+            while(cur_NNZ < cur_bound && cur_row < CRS->m) {
+                ++cur_row;
+                cur_NNZ += CRS->rowptr[cur_row+1] - CRS->rowptr[cur_row];
+            }
+            CRS->sliceptr[slice] = cur_row;
+            cur_bound += slice_size;
+        }
+        CRS->sliceptr[CRS->nslices]=CRS->m;
     }
 }
 
@@ -403,45 +403,44 @@ void nlCRSMatrixPatternCompile(NLCRSMatrix* M) {
     NLuint nslices = nlGetNumThreads();
     NLuint i;
     NLuint_big nnz;
-    NLuint k;
+    NLuint_big k;
     /* Test that matrix is in 'pattern' state */
     nl_assert(M->colind == NULL);
     nl_assert(M->val == NULL);
     for(i=0; i<M->m; ++i) {
-	M->rowptr[i+1] += M->rowptr[i];
+        M->rowptr[i+1] += M->rowptr[i];
     }
     nnz = M->rowptr[M->m];
     M->val = NL_NEW_ARRAY(double, nnz);
     M->colind = NL_NEW_ARRAY(NLuint, nnz);
     for(k=0; k<nnz; ++k) {
-	M->colind[k] = (NLuint)(-1);
+        M->colind[k] = (NLuint)(-1);
     }
     M->sliceptr = NL_NEW_ARRAY(NLuint, nslices+1);
     M->nslices  = nslices;
     nlCRSMatrixComputeSlices(M);
 }
 
-void nlCRSMatrixAdd(
+NLulong nlCRSMatrixAdd(
     NLCRSMatrix* M, NLuint i, NLuint j, NLdouble value
 ) {
-    NLuint_big jj;    
-    /* Test that matrix is in 'compiled' state */
-    nl_assert(M->colind != NULL);
-    nl_assert(M->val != NULL);
-    nl_assert(i < M->m);
-    nl_assert(j < M->n);
+    NLuint_big jj;
+    nl_debug_assert(M->colind != NULL);
+    nl_debug_assert(M->val != NULL);
+    nl_debug_assert(i < M->m);
+    nl_debug_assert(j < M->n);
     if(M->symmetric_storage && j > i) {
-	return;
+        return (NLulong)(-1);
     }
     for(jj=M->rowptr[i]; jj<M->rowptr[i+1]; ++jj) {
-	if(M->colind[jj] == j) {
-	    M->val[jj] += value;
-	    return;
-	} else if(M->colind[jj] == (NLuint)(-1)) {
-	    M->colind[jj] = j;
-	    M->val[jj] += value;
-	    return;
-	}
+        if(M->colind[jj] == j) {
+            M->val[jj] += value;
+            return jj;
+        } else if(M->colind[jj] == (NLuint)(-1)) {
+            M->colind[jj] = j;
+            M->val[jj] += value;
+            return jj;
+        }
     }
     /* If this line is reached, it means that too many coefficients
      * were added to row j, i.e. a number of coefficients larger than
@@ -450,6 +449,39 @@ void nlCRSMatrixAdd(
     nl_assert_not_reached;
 }
 
+void nlCRSMatrixAddAt(
+    NLCRSMatrix* M, NLuint i, NLuint j, NLdouble value, NLulong index
+) {
+    nl_arg_used(i);
+    nl_arg_used(j);
+    nl_debug_assert(M->colind != NULL);
+    nl_debug_assert(M->val != NULL);
+    nl_debug_assert(i < M->m);
+    nl_debug_assert(j < M->n);
+    nl_debug_assert(index != (NLulong)(-1));
+    nl_debug_assert(index < M->rowptr[M->m]);
+    nl_debug_assert(M->colind[index] == j);
+    nl_debug_assert(index >= M->rowptr[i] && index < M->rowptr[i+1]);
+    M->val[index] += value;
+}
+
+void nlCRSMatrixSetCoefficientAtRowOffset(
+    NLCRSMatrix* M, NLuint i, NLuint j, NLdouble value, NLuint row_offset
+) {
+    NLuint_big index = (NLuint_big)i + (NLuint_big)row_offset;
+    nl_arg_used(i);
+    nl_arg_used(j);
+    nl_debug_assert(M->colind != NULL);
+    nl_debug_assert(M->val != NULL);
+    nl_debug_assert(i < M->m);
+    nl_debug_assert(j < M->n);
+    nl_debug_assert(index < M->rowptr[M->m]);
+    nl_debug_assert(M->colind[index] == (NLuint)(-1));
+    nl_debug_assert(index >= M->rowptr[i] && index < M->rowptr[i+1]);
+    M->val[index] = value;
+    M->colind[index] = j;
+}
+
 /******************************************************************************/
 /* SparseMatrix data structure */
 
@@ -463,14 +495,14 @@ static void nlSparseMatrixDestroyRowColumns(NLSparseMatrix* M) {
         NL_DELETE_ARRAY(M->row);
     }
     M->storage = (NLenum)((int)(M->storage) & ~NL_MATRIX_STORE_ROWS);
-    
+
     if(M->storage & NL_MATRIX_STORE_COLUMNS) {
         for(i=0; i<M->n; i++) {
             nlRowColumnDestroy(&(M->column[i]));
         }
         NL_DELETE_ARRAY(M->column);
     }
-    M->storage = (NLenum)((int)(M->storage) & ~NL_MATRIX_STORE_COLUMNS);    
+    M->storage = (NLenum)((int)(M->storage) & ~NL_MATRIX_STORE_COLUMNS);
 }
 
 void nlSparseMatrixDestroy(NLSparseMatrix* M) {
@@ -500,54 +532,54 @@ void nlSparseMatrixAdd(NLSparseMatrix* M, NLuint i, NLuint j, NLdouble value) {
 }
 
 static void nlSparseMatrixAddSparseMatrix(
-    NLSparseMatrix* M, double mul, const NLSparseMatrix* N    
+    NLSparseMatrix* M, double mul, const NLSparseMatrix* N
 ) {
     NLuint i,j,ii,jj;
     nl_assert(M->m == N->m);
     nl_assert(M->n == N->n);
     if(N->storage & NL_MATRIX_STORE_SYMMETRIC) {
-	nl_assert(M->storage & NL_MATRIX_STORE_SYMMETRIC);
+        nl_assert(M->storage & NL_MATRIX_STORE_SYMMETRIC);
     }
     if(N->storage & NL_MATRIX_STORE_ROWS) {
-	for(i=0; i<N->m; ++i) {
-	    for(jj=0; jj<N->row[i].size; ++jj) {
-		nlSparseMatrixAdd(
-		    M,
-		    i, N->row[i].coeff[jj].index,
-		    mul*N->row[i].coeff[jj].value
-		);
-	    }
-	}
+        for(i=0; i<N->m; ++i) {
+            for(jj=0; jj<N->row[i].size; ++jj) {
+                nlSparseMatrixAdd(
+                    M,
+                    i, N->row[i].coeff[jj].index,
+                    mul*N->row[i].coeff[jj].value
+                );
+            }
+        }
     } else {
-	nl_assert(N->storage & NL_MATRIX_STORE_COLUMNS);	
-	for(j=0; j<N->n; ++j) {
-	    for(ii=0; ii<N->column[j].size; ++ii) {
-		nlSparseMatrixAdd(
-		    M,
-		    N->column[j].coeff[ii].index, j,
-		    mul*N->column[j].coeff[ii].value
-		);
-	    }
-	}
+        nl_assert(N->storage & NL_MATRIX_STORE_COLUMNS);
+        for(j=0; j<N->n; ++j) {
+            for(ii=0; ii<N->column[j].size; ++ii) {
+                nlSparseMatrixAdd(
+                    M,
+                    N->column[j].coeff[ii].index, j,
+                    mul*N->column[j].coeff[ii].value
+                );
+            }
+        }
     }
 }
 
 static void nlSparseMatrixAddCRSMatrix(
-    NLSparseMatrix* M, double mul, const NLCRSMatrix* N    
+    NLSparseMatrix* M, double mul, const NLCRSMatrix* N
 ) {
     NLuint i;
     NLuint_big jj;
     nl_assert(M->m == N->m);
     nl_assert(M->n == N->n);
     for(i=0; i<M->m; ++i) {
-	for(jj=N->rowptr[i]; jj<N->rowptr[i+1]; ++jj) {
-	    nlSparseMatrixAdd(
-		M,
-		i,
-		N->colind[jj],
-		mul*N->val[jj]
-	    );
-	}
+        for(jj=N->rowptr[i]; jj<N->rowptr[i+1]; ++jj) {
+            nlSparseMatrixAdd(
+                M,
+                i,
+                N->colind[jj],
+                mul*N->val[jj]
+            );
+        }
     }
 }
 
@@ -557,14 +589,14 @@ void nlSparseMatrixAddMatrix(
     nl_assert(M->m == N->m);
     nl_assert(M->n == N->n);
     if(N->type == NL_MATRIX_SPARSE_DYNAMIC) {
-	nlSparseMatrixAddSparseMatrix(M, mul, (const NLSparseMatrix*)N);
+        nlSparseMatrixAddSparseMatrix(M, mul, (const NLSparseMatrix*)N);
     } else if(N->type == NL_MATRIX_CRS) {
-	nlSparseMatrixAddCRSMatrix(M, mul, (const NLCRSMatrix*)N);	
+        nlSparseMatrixAddCRSMatrix(M, mul, (const NLCRSMatrix*)N);
     } else {
-	nl_assert_not_reached;
+        nl_assert_not_reached;
     }
 }
-    
+
 
 
 void nlSparseMatrixZero( NLSparseMatrix* M) {
@@ -619,14 +651,14 @@ void nlSparseMatrixSort( NLSparseMatrix* M) {
     NLuint i;
     if(M->storage & NL_MATRIX_STORE_ROWS) {
         for(i = 0; i<M->m; i++) {
-            nlRowColumnSort(&(M->row[i]));                
+            nlRowColumnSort(&(M->row[i]));
         }
-    } 
+    }
     if (M->storage & NL_MATRIX_STORE_COLUMNS) {
         for(i = 0; i<M->n; i++) {
             nlRowColumnSort(&(M->column[i]));
         }
-    } 
+    }
 }
 
 void nlSparseMatrixMAddRow(
@@ -638,10 +670,10 @@ void nlSparseMatrixMAddRow(
 
     nl_debug_assert(i1 < M->m);
     nl_debug_assert(i2 < M->m);
-    
+
     for(jj=0; jj<Ri2->size; ++jj) {
-	c = &(Ri2->coeff[jj]);
-	nlSparseMatrixAdd(M, i1, c->index, s*c->value);
+        c = &(Ri2->coeff[jj]);
+        nlSparseMatrixAdd(M, i1, c->index, s*c->value);
     }
 }
 
@@ -653,15 +685,15 @@ void nlSparseMatrixScaleRow(
     NLCoeff* c = NULL;
 
     nl_assert(M->storage & NL_MATRIX_STORE_ROWS);
-    nl_assert(!(M->storage & NL_MATRIX_STORE_COLUMNS));    
+    nl_assert(!(M->storage & NL_MATRIX_STORE_COLUMNS));
     nl_debug_assert(i < M->m);
-    
+
     for(jj=0; jj<Ri->size; ++jj) {
-	c = &(Ri->coeff[jj]);
-	c->value *= s;
+        c = &(Ri->coeff[jj]);
+        c->value *= s;
     }
     if(i < M->diag_size) {
-	M->diag[i] *= s;
+        M->diag[i] *= s;
     }
 }
 
@@ -671,10 +703,10 @@ void nlSparseMatrixZeroRow(
     NLRowColumn* Ri = &(M->row[i]);
 
     nl_debug_assert(i < M->m);
-    
+
     Ri->size = 0;
     if(i < M->diag_size) {
-	M->diag[i] = 0.0;
+        M->diag[i] = 0.0;
     }
 }
 
@@ -704,28 +736,28 @@ static void nlSparseMatrix_mult_rows_symmetric(
 }
 
 static void nlSparseMatrix_mult_rows(
-        NLSparseMatrix* A,
-        const NLdouble* x,
-        NLdouble* y
+    NLSparseMatrix* A,
+    const NLdouble* x,
+    NLdouble* y
 ) {
-    /* 
+    /*
      * Note: OpenMP does not like unsigned ints
      * (causes some floating point exceptions),
      * therefore I use here signed ints for all
      * indices.
      */
-    
+
     int m = (int)(A->m);
     int i,ij;
     NLCoeff* c = NULL;
     NLRowColumn* Ri = NULL;
 
-#if defined(_OPENMP)    
+#if defined(_OPENMP)
 #pragma omp parallel for private(i,ij,c,Ri)
 #endif
-    
+
     for(i=0; i<m; i++) {
-        Ri = &(A->row[i]);       
+        Ri = &(A->row[i]);
         y[i] = 0;
         for(ij=0; ij<(int)(Ri->size); ij++) {
             c = &(Ri->coeff[ij]);
@@ -735,15 +767,15 @@ static void nlSparseMatrix_mult_rows(
 }
 
 static void nlSparseMatrix_mult_cols_symmetric(
-        NLSparseMatrix* A,
-        const NLdouble* x,
-        NLdouble* y
+    NLSparseMatrix* A,
+    const NLdouble* x,
+    NLdouble* y
 ) {
     NLuint n = A->n;
     NLuint j,ii;
     NLCoeff* c = NULL;
     for(j=0; j<n; j++) {
-        NLRowColumn* Cj = &(A->column[j]);       
+        NLRowColumn* Cj = &(A->column[j]);
         y[j] = 0;
         for(ii=0; ii<Cj->size; ii++) {
             c = &(Cj->coeff[ii]);
@@ -756,12 +788,12 @@ static void nlSparseMatrix_mult_cols_symmetric(
 }
 
 static void nlSparseMatrix_mult_cols(
-        NLSparseMatrix* A,
-        const NLdouble* x,
-        NLdouble* y
+    NLSparseMatrix* A,
+    const NLdouble* x,
+    NLdouble* y
 ) {
     NLuint n = A->n;
-    NLuint j,ii; 
+    NLuint j,ii;
     NLCoeff* c = NULL;
     NL_CLEAR_ARRAY(NLdouble, y, A->m);
     for(j=0; j<n; j++) {
@@ -813,24 +845,24 @@ void nlSparseMatrixConstruct(
     M->storage = storage;
     if(storage & NL_MATRIX_STORE_ROWS) {
         M->row = NL_NEW_ARRAY(NLRowColumn, m);
-	M->row_capacity = m;
+        M->row_capacity = m;
         for(i=0; i<n; i++) {
             nlRowColumnConstruct(&(M->row[i]));
         }
     } else {
         M->row = NULL;
-	M->row_capacity = 0;
+        M->row_capacity = 0;
     }
 
     if(storage & NL_MATRIX_STORE_COLUMNS) {
         M->column = NL_NEW_ARRAY(NLRowColumn, n);
-	M->column_capacity = n;
+        M->column_capacity = n;
         for(i=0; i<n; i++) {
             nlRowColumnConstruct(&(M->column[i]));
         }
     } else {
         M->column = NULL;
-	M->column_capacity = 0;
+        M->column_capacity = 0;
     }
 
     M->diag_size = MIN(m,n);
@@ -839,7 +871,7 @@ void nlSparseMatrixConstruct(
 }
 
 /**
- * \brief Adjusts the size of the diagonal of 
+ * \brief Adjusts the size of the diagonal of
  *  an NLSparseMatrix after the number of rows or c
  *  olumns have changed.
  * \param[in,out] M a pointer to the sparse matrix.
@@ -848,33 +880,33 @@ static void adjust_diag(NLSparseMatrix* M) {
     NLuint new_diag_size = MIN(M->m, M->n);
     NLuint i;
     if(new_diag_size > M->diag_size) {
-	if(new_diag_size > M->diag_capacity) {
-	    M->diag_capacity *= 2;
-	    if(M->diag_capacity == 0) {
-		M->diag_capacity = 16;
-	    }
-	    M->diag = NL_RENEW_ARRAY(double, M->diag, M->diag_capacity);
-	    for(i=M->diag_size; i<new_diag_size; ++i) {
-		M->diag[i] = 0.0;
-	    }
-	}
-	M->diag_size= new_diag_size;
+        if(new_diag_size > M->diag_capacity) {
+            M->diag_capacity *= 2;
+            if(M->diag_capacity == 0) {
+                M->diag_capacity = 16;
+            }
+            M->diag = NL_RENEW_ARRAY(double, M->diag, M->diag_capacity);
+            for(i=M->diag_size; i<new_diag_size; ++i) {
+                M->diag[i] = 0.0;
+            }
+        }
+        M->diag_size= new_diag_size;
     }
 }
 
 void nlSparseMatrixAddRow( NLSparseMatrix* M) {
     ++M->m;
     if(M->storage & NL_MATRIX_STORE_ROWS) {
-	if(M->m > M->row_capacity) {
-	    M->row_capacity *= 2;
-	    if(M->row_capacity == 0) {
-		M->row_capacity = 16;
-	    }
-	    M->row = NL_RENEW_ARRAY(
-		NLRowColumn, M->row, M->row_capacity
-	    );
-	}
-	nlRowColumnConstruct(&(M->row[M->m-1]));
+        if(M->m > M->row_capacity) {
+            M->row_capacity *= 2;
+            if(M->row_capacity == 0) {
+                M->row_capacity = 16;
+            }
+            M->row = NL_RENEW_ARRAY(
+                NLRowColumn, M->row, M->row_capacity
+            );
+        }
+        nlRowColumnConstruct(&(M->row[M->m-1]));
     }
     adjust_diag(M);
 }
@@ -882,16 +914,16 @@ void nlSparseMatrixAddRow( NLSparseMatrix* M) {
 void nlSparseMatrixAddColumn( NLSparseMatrix* M) {
     ++M->n;
     if(M->storage & NL_MATRIX_STORE_COLUMNS) {
-	if(M->n > M->column_capacity) {
-	    M->column_capacity *= 2;
-	    if(M->column_capacity == 0) {
-		M->column_capacity = 16;
-	    }
-	    M->column = NL_RENEW_ARRAY(
-		NLRowColumn, M->column, M->column_capacity
-	    );
-	}
-	nlRowColumnConstruct(&(M->column[M->n-1]));
+        if(M->n > M->column_capacity) {
+            M->column_capacity *= 2;
+            if(M->column_capacity == 0) {
+                M->column_capacity = 16;
+            }
+            M->column = NL_RENEW_ARRAY(
+                NLRowColumn, M->column, M->column_capacity
+            );
+        }
+        nlRowColumnConstruct(&(M->column[M->n-1]));
     }
     adjust_diag(M);
 }
@@ -902,18 +934,18 @@ void nlSparseMatrixAddColumn( NLSparseMatrix* M) {
 NLMatrix nlCRSMatrixNewFromSparseMatrix(NLSparseMatrix* M) {
     NLuint_big nnz = nlSparseMatrixNNZ(M);
     NLuint nslices = nlGetNumThreads();
-    NLuint i,ij,k; 
+    NLuint i,ij,k;
     NLCRSMatrix* CRS = NL_NEW(NLCRSMatrix);
 
     nl_assert(M->storage & NL_MATRIX_STORE_ROWS);
 
     if(M->storage & NL_MATRIX_STORE_SYMMETRIC) {
         nl_assert(M->m == M->n);
-        nlCRSMatrixConstructSymmetric(CRS, M->n, nnz);        
+        nlCRSMatrixConstructSymmetric(CRS, M->n, nnz);
     } else {
         nlCRSMatrixConstruct(CRS, M->m, M->n, nnz, nslices);
     }
-    
+
     nlSparseMatrixSort(M);
     /* Convert matrix to CRS format */
     k=0;
@@ -936,12 +968,12 @@ NLMatrix nlCRSMatrixNewFromSparseMatrixSymmetric(NLSparseMatrix* M) {
     NLuint_big nnz;
     NLuint i,j,jj,k;
     NLCRSMatrix* CRS = NL_NEW(NLCRSMatrix);
-    
+
     nl_assert(M->storage & NL_MATRIX_STORE_ROWS);
     nl_assert(M->m == M->n);
 
     nlSparseMatrixSort(M);
-    
+
     if(M->storage & NL_MATRIX_STORE_SYMMETRIC) {
         nnz = nlSparseMatrixNNZ(M);
     } else {
@@ -957,7 +989,7 @@ NLMatrix nlCRSMatrixNewFromSparseMatrixSymmetric(NLSparseMatrix* M) {
         }
     }
 
-    nlCRSMatrixConstructSymmetric(CRS, M->n, nnz);        
+    nlCRSMatrixConstructSymmetric(CRS, M->n, nnz);
 
     k=0;
     for(i=0; i<M->m; ++i) {
@@ -983,25 +1015,25 @@ NLMatrix nlCRSMatrixNewFromSparseMatrixSymmetric(NLSparseMatrix* M) {
 
 void nlMatrixCompress(NLMatrix* M) {
     NLMatrix result = NULL;
-    
+
     if(
-	(*M)->type == NL_MATRIX_CRS &&
-	nlExtensionIsInitialized_MKL()
+        (*M)->type == NL_MATRIX_CRS &&
+        nlExtensionIsInitialized_MKL()
     ) {
-	result = nlMKLMatrixNewFromCRSMatrix((NLCRSMatrix*)*M);
-	nlDeleteMatrix(*M);
-	*M = result;
-	return;
+        result = nlMKLMatrixNewFromCRSMatrix((NLCRSMatrix*)*M);
+        nlDeleteMatrix(*M);
+        *M = result;
+        return;
     }
-    
+
     if((*M)->type != NL_MATRIX_SPARSE_DYNAMIC) {
         return;
     }
-    
+
     if(nlExtensionIsInitialized_MKL()) {
-	result = nlMKLMatrixNewFromSparseMatrix((NLSparseMatrix*)*M);
+        result = nlMKLMatrixNewFromSparseMatrix((NLSparseMatrix*)*M);
     } else {
-	result = nlCRSMatrixNewFromSparseMatrix((NLSparseMatrix*)*M);
+        result = nlCRSMatrixNewFromSparseMatrix((NLSparseMatrix*)*M);
     }
     nlDeleteMatrix(*M);
     *M = result;
@@ -1009,9 +1041,9 @@ void nlMatrixCompress(NLMatrix* M) {
 
 NLuint_big nlMatrixNNZ(NLMatrix M) {
     if(M->type == NL_MATRIX_SPARSE_DYNAMIC) {
-	return nlSparseMatrixNNZ((NLSparseMatrix*)M);
+        return nlSparseMatrixNNZ((NLSparseMatrix*)M);
     } else if(M->type == NL_MATRIX_CRS) {
-	return nlCRSMatrixNNZ((NLCRSMatrix*)M);	
+        return nlCRSMatrixNNZ((NLCRSMatrix*)M);
     }
     return (NLuint_big)(M->m) * (NLuint_big)(M->n);
 }
@@ -1019,16 +1051,16 @@ NLuint_big nlMatrixNNZ(NLMatrix M) {
 NLMatrix nlMatrixFactorize(NLMatrix M, NLenum solver) {
     NLMatrix result = NULL;
     switch(solver) {
-	case NL_SUPERLU_EXT:
-	case NL_PERM_SUPERLU_EXT:      
-	case NL_SYMMETRIC_SUPERLU_EXT:
-	    result = nlMatrixFactorize_SUPERLU(M,solver);
-	    break;
-	case NL_CHOLMOD_EXT:
-	    result = nlMatrixFactorize_CHOLMOD(M,solver);	    
-	    break;
-	default:
-	    nlError("nlMatrixFactorize","unknown solver");
+    case NL_SUPERLU_EXT:
+    case NL_PERM_SUPERLU_EXT:
+    case NL_SYMMETRIC_SUPERLU_EXT:
+        result = nlMatrixFactorize_SUPERLU(M,solver);
+        break;
+    case NL_CHOLMOD_EXT:
+        result = nlMatrixFactorize_CHOLMOD(M,solver);
+        break;
+    default:
+        nlError("nlMatrixFactorize","unknown solver");
     }
     return result;
 }
@@ -1040,18 +1072,18 @@ NLMatrix nlMatrixFactorize(NLMatrix M, NLenum solver) {
  */
 typedef struct {
     /**
-     * \brief number of rows 
-     */    
+     * \brief number of rows
+     */
     NLuint m;
 
     /**
-     * \brief number of columns 
-     */    
+     * \brief number of columns
+     */
     NLuint n;
 
     /**
      * \brief Matrix type
-     * \details One of NL_MATRIX_SPARSE_DYNAMIC, 
+     * \details One of NL_MATRIX_SPARSE_DYNAMIC,
      *  NL_MATRIX_CRS, NL_MATRIX_SUPERLU_EXT,
      *  NL_MATRIX_CHOLDMOD_EXT, NL_MATRIX_FUNCTION,
      *  NL_MATRIX_OTHER
@@ -1078,8 +1110,8 @@ typedef struct {
 
 static void nlFunctionMatrixDestroy(NLFunctionMatrix* M) {
     (void)M; /* to avoid 'unused parameter' warning */
-    /* 
-     * Nothing special to do, 
+    /*
+     * Nothing special to do,
      * there is no dynamic allocated mem.
      */
 }
@@ -1103,10 +1135,10 @@ NLMatrix nlMatrixNewFromFunction(NLuint m, NLuint n, NLMatrixFunc func) {
 
 NLMatrixFunc nlMatrixGetFunction(NLMatrix M) {
     if(M == NULL) {
-	return NULL;
+        return NULL;
     }
     if(M->type != NL_MATRIX_FUNCTION) {
-	return NULL;
+        return NULL;
     }
     return ((NLFunctionMatrix*)M)->matrix_func;
 }
@@ -1118,13 +1150,13 @@ NLMatrixFunc nlMatrixGetFunction(NLMatrix M) {
  */
 typedef struct {
     /**
-     * \brief number of rows 
-     */    
+     * \brief number of rows
+     */
     NLuint m;
 
     /**
-     * \brief number of columns 
-     */    
+     * \brief number of columns
+     */
     NLuint n;
 
     /**
@@ -1159,7 +1191,7 @@ typedef struct {
      *  NL_FALSE otherwise.
      */
     NLboolean owns_M;
-    
+
     /**
      * \brief Second matrix of the product.
      */
@@ -1170,7 +1202,7 @@ typedef struct {
      *  NL_FALSE otherwise.
      */
     NLboolean owns_N;
-    
+
     /**
      * \brief A temporary vector of dimension N->m (= M->n)
      */
@@ -1181,10 +1213,10 @@ typedef struct {
 static void nlMatrixProductDestroy(NLMatrixProduct* P) {
     NL_DELETE_ARRAY(P->work);
     if(P->owns_M) {
-	nlDeleteMatrix(P->M); P->M = NULL;
+        nlDeleteMatrix(P->M); P->M = NULL;
     }
     if(P->owns_N) {
-	nlDeleteMatrix(P->N); P->N = NULL;
+        nlDeleteMatrix(P->N); P->N = NULL;
     }
 }
 
