@@ -41,7 +41,6 @@
 #ifndef GEOGRAM_BASIC_ATTRIBUTES
 #define GEOGRAM_BASIC_ATTRIBUTES
 
-
 #include <geogram/basic/common.h>
 #include <geogram/basic/memory.h>
 #include <geogram/basic/numeric.h>
@@ -119,6 +118,14 @@ namespace GEO {
         return size_ * dimension_;
     }
 
+    /**
+     * \brief Gets a pointer to the storage
+     * \return a pointer to the first element
+     */
+    Memory::pointer base_addr() const {
+	return base_addr_;
+    }
+
     /**
      * \brief Registers this observer to an AttributeStore.
      * \param[in] store a pointer to the AttributeStore.
@@ -145,6 +152,16 @@ namespace GEO {
         disconnected_ = true;
     }
 
+    /**
+     * \brief Tests whether this AttributeStoreObserver was disconnected
+     * \retval true if this AttributeStoreObserver was disconnected,
+     * \retval false otherwise
+     * \see disconnect()
+     */
+    bool disconnected() const {
+	return disconnected_;
+    }
+
     protected:
     Memory::pointer base_addr_;
     index_t size_;
@@ -203,7 +220,7 @@ namespace GEO {
      *  attributes and can be greater for vector
      *  attributes.
      */
-    AttributeStore(index_t elemsize, index_t dim=1);
+    AttributeStore(size_t elemsize, index_t dim=1);
 
     /**
      * \brief AttributeStore destructor.
@@ -363,7 +380,7 @@ namespace GEO {
     void copy_item(index_t to, index_t from) {
         geo_debug_assert(from < cached_size_);
         geo_debug_assert(to < cached_size_);
-        index_t item_size = element_size_ * dimension_;
+        size_t item_size = element_size_ * dimension_;
         Memory::copy(
             cached_base_addr_+to*item_size,
             cached_base_addr_+from*item_size,
@@ -377,7 +394,7 @@ namespace GEO {
      */
     void zero_item(index_t to) {
         geo_debug_assert(to < cached_size_);
-        index_t item_size = element_size_ * dimension_;
+        size_t item_size = element_size_ * dimension_;
         Memory::clear(
             cached_base_addr_+to*item_size,
             item_size
@@ -430,7 +447,7 @@ namespace GEO {
      * \return the size of an element, in bytes
      */
     size_t element_size() const {
-        return size_t(element_size_);
+        return element_size_;
     }
 
     /**
@@ -525,23 +542,7 @@ namespace GEO {
         AttributeStoreCreator* creator,
         const std::string& element_type_name,
         const std::string& element_typeid_name
-    ) {
-        if(element_type_name_is_known(element_type_name)) {
-            Logger::warn("Attributes") << element_type_name
-                                       << " already registered"
-                                       << std::endl;
-            if(element_typeid_name_is_known(element_typeid_name)) {
-                bool already_registered_attribute_has_same_type = (
-                    type_name_to_typeid_name_[element_type_name] ==
-                    element_typeid_name
-                );
-                geo_assert(already_registered_attribute_has_same_type);
-            }
-        }
-        type_name_to_creator_[element_type_name] = creator;
-        typeid_name_to_type_name_[element_typeid_name] = element_type_name;
-        type_name_to_typeid_name_[element_type_name] = element_typeid_name;
-    }
+    );
 
     protected:
     /**
@@ -577,7 +578,7 @@ namespace GEO {
 
 
     protected:
-    index_t element_size_;
+    size_t element_size_;
     index_t dimension_;
     Memory::pointer cached_base_addr_;
     index_t cached_size_;
@@ -614,7 +615,7 @@ namespace GEO {
          *  attributes.
          */
         TypedAttributeStore(index_t dim=1) :
-            AttributeStore(index_t(sizeof(T)),dim) {
+            AttributeStore(sizeof(T),dim) {
         }
 
         void resize(index_t new_size) override {
@@ -668,9 +669,7 @@ namespace GEO {
             );
         }
 
-        bool elements_type_matches(
-            const std::string& type_name
-        ) const override {
+        bool elements_type_matches(const std::string& type_name) const override {
             return type_name == typeid(T).name();
         }
 
@@ -709,8 +708,8 @@ namespace GEO {
 
     protected:
         void notify(
-            Memory::pointer base_addr, index_t size, index_t dim
-        ) override {
+	    Memory::pointer base_addr, index_t size, index_t dim
+	) override {
             AttributeStore::notify(base_addr, size, dim);
             geo_assert(size*dim <= store_.size());
         }
@@ -1080,7 +1079,7 @@ namespace GEO {
      *   instance by passing it by-value to a function).
      *   Use copy() instead.
      */
-    AttributesManager(const AttributesManager& rhs);
+    AttributesManager(const AttributesManager& rhs) = delete;
 
     /**
      * \brief Forbids copy.
@@ -1089,7 +1088,7 @@ namespace GEO {
      *   instance by passing it by-value to a function).
      *   Use copy() instead.
      */
-    const AttributesManager& operator=(const AttributesManager& rhs);
+    const AttributesManager& operator=(const AttributesManager& rhs) = delete;
 
     private:
     index_t size_;
@@ -1203,7 +1202,7 @@ namespace GEO {
 
         /**
          * \brief Binds this Attribute to an AttributesManager if it
-         *  already exists in the AttributesManager and tyopes are compatible.
+         *  already exists in the AttributesManager and types are compatible.
          * \param[in] manager a reference to the AttributesManager
          * \param[in] name name of the attribute
          * \pre !is_bound()
@@ -1444,6 +1443,7 @@ namespace GEO {
          * \return a modifiable reference to the \p i%th element
          */
         T& operator[](index_t i) {
+	    geo_debug_assert(superclass::is_bound());
             geo_debug_assert(i < superclass::nb_elements());
             return ((T*)(void*)superclass::base_addr_)[i];
         }
@@ -1454,6 +1454,7 @@ namespace GEO {
          * \return a const reference to the \p i%th element
          */
         const T& operator[](index_t i) const {
+	    geo_debug_assert(superclass::is_bound());
             geo_debug_assert(i < superclass::nb_elements());
             return ((const T*)(void*)superclass::base_addr_)[i];
         }
@@ -1464,6 +1465,7 @@ namespace GEO {
          * \param[in] val the value
          */
         void fill(const T& val) {
+	    geo_debug_assert(superclass::is_bound());
             for(index_t i=0; i<superclass::nb_elements(); ++i) {
                 (*this)[i] = val;
             }
@@ -1504,11 +1506,11 @@ namespace GEO {
         /**
          * \brief Forbids copy.
          */
-        Attribute(const Attribute<T>& rhs);
+        Attribute(const Attribute<T>& rhs) = delete;
         /**
          * \brief Forbids copy.
          */
-        Attribute<T>& operator=(const Attribute<T>& rhs);
+        Attribute<T>& operator=(const Attribute<T>& rhs) = delete;
     };
 
     /*********************************************************************/
@@ -1664,6 +1666,20 @@ namespace GEO {
             }
         }
 
+        /**
+         * \brief Copies all the values from another attribute.
+         * \param[in] rhs the attribute to be copied.
+         * \details rhs needs to have the same size and dimension
+         *  as this Attribute.
+         */
+        void copy(const Attribute<bool>& rhs) {
+            geo_assert(rhs.size() == superclass::size());
+            geo_assert(rhs.dimension() == superclass::dimension());
+            for(index_t i=0; i<superclass::nb_elements(); ++i) {
+		element(i) = rhs.element(i);
+            }
+        }
+
     protected:
 
         friend class BoolAttributeAccessor;
@@ -1693,11 +1709,11 @@ namespace GEO {
         /**
          * \brief Forbids copy.
          */
-        Attribute(const Attribute<bool>& rhs);
+        Attribute(const Attribute<bool>& rhs) = delete;
         /**
          * \brief Forbids copy.
          */
-        Attribute<bool>& operator=(const Attribute<bool>& rhs);
+        Attribute<bool>& operator=(const Attribute<bool>& rhs) = delete;
     } ;
 
     /***********************************************************/
