@@ -724,9 +724,13 @@ namespace {
         declare_arg("gui:font_size", 18, "font size");
         declare_arg("gui:expert", false, "expert mode for developpers");
 #ifdef GEO_OS_ANDROID
-        declare_arg("gui:phone_screen", true, "running on a phone (or testing)");
+        declare_arg(
+	    "gui:phone_screen", true, "running on a phone (or testing)"
+	);
 #else
-        declare_arg("gui:phone_screen", false, "running on a phone (or testing)");
+        declare_arg(
+	    "gui:phone_screen", false, "running on a phone (or testing)"
+	);
 #endif
     }
 
