@@ -13,7 +13,7 @@
  *  * Neither the name of the ALICE Project-Team nor the names of its
  *  contributors may be used to endorse or promote products derived from this
  *  software without specific prior written permission.
- * 
+ *
  *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -42,20 +42,64 @@
 #include <geogram/basic/command_line.h>
 #include <geogram/basic/argused.h>
 #include <geogram/basic/file_system.h>
+#include <geogram/basic/process.h>
+
 
 #include <stdlib.h>
 #include <stdarg.h>
 
 /*
-   Disables the warning caused by passing 'this' as an argument while
-   construction is not finished (in LoggerStream ctor).
-   As LoggerStreamBuf only stores the pointer for later use, so we can
-   ignore the fact that 'this' is not completely formed yet.
- */
+  Disables the warning caused by passing 'this' as an argument while
+  construction is not finished (in LoggerStream ctor).
+  As LoggerStreamBuf only stores the pointer for later use, so we can
+  ignore the fact that 'this' is not completely formed yet.
+*/
 #ifdef GEO_OS_WINDOWS
 #pragma warning(disable:4355)
 #endif
 
+
+namespace {
+    using namespace GEO;
+
+    /**
+     * \brief The output stream returned by Logger::err_console()
+     * \details It has a locking mechanism that avoids messages sent by different
+     *  threads to be mixed.
+     * \see Logger::err_console(), Logger::out(), Logger::err(), Logger::warn(),
+     *  Logger::status()
+     */
+    class CERRStream : public std::ostream {
+    public:
+        CERRStream() :
+            std::ostream(new CERRStreamBuff(this)),lock_(GEOGRAM_SPINLOCK_INIT) {
+        }
+        ~CERRStream() override{
+        }
+        void lock() {
+            Process::acquire_spinlock(lock_);
+        }
+        void unlock() {
+            Process::release_spinlock(lock_);
+        }
+    private:
+        Process::spinlock lock_;
+        class CERRStreamBuff : public std::stringbuf {
+        public:
+            CERRStreamBuff(CERRStream* stream) : stream_(stream) {
+            }
+            int sync() override {
+                std::cerr << this->str();
+                this->str("");
+                stream_->unlock();
+                return 0;
+            }
+        private:
+            CERRStream* stream_;
+        };
+    };
+}
+
 namespace GEO {
 
     /************************************************************************/
@@ -80,7 +124,7 @@ namespace GEO {
     }
 
     void LoggerStream::notify(const std::string& str) {
-	logger_->notify(this, str);
+        logger_->notify(this, str);
     }
 
     /************************************************************************/
@@ -190,7 +234,7 @@ namespace GEO {
     bool Logger::is_initialized() {
         return (instance_ != nullptr);
     }
-    
+
     bool Logger::set_local_value(
         const std::string& name, const std::string& value
     ) {
@@ -204,7 +248,7 @@ namespace GEO {
             set_minimal(String::to_bool(value));
             return true;
         }
-        
+
         if(name == "log:pretty") {
             set_pretty(String::to_bool(value));
             return true;
@@ -261,7 +305,7 @@ namespace GEO {
             value = String::to_string(is_minimal());
             return true;
         }
-        
+
         if(name == "log:pretty") {
             value = String::to_string(is_pretty());
             return true;
@@ -307,7 +351,7 @@ namespace GEO {
 
     void Logger::unregister_client(LoggerClient* c) {
         geo_debug_assert(clients_.find(c) != clients_.end());
-	clients_.erase(c);
+        clients_.erase(c);
     }
 
     void Logger::unregister_all_clients() {
@@ -325,7 +369,7 @@ namespace GEO {
     void Logger::set_minimal(bool flag) {
         minimal_ = flag;
     }
-    
+
     void Logger::set_pretty(bool flag) {
         pretty_ = flag;
     }
@@ -341,21 +385,25 @@ namespace GEO {
         quiet_(true),
         pretty_(true),
         minimal_(false),
-	notifying_error_(false)
+        notifying_error_(false),
+	indent_(0)
     {
         // Add a default client printing stuff to std::cout
         register_client(new ConsoleLogger());
 #ifdef GEO_DEBUG
         quiet_ = false;
-#endif        
+#endif
+        err_console_ = new CERRStream;
     }
 
     Logger::~Logger() {
+        delete err_console_;
+        err_console_ = nullptr;
     }
 
     Logger* Logger::instance() {
         // Do not use geo_assert here:
-	//  if the instance is nullptr, geo_assert will
+        //  if the instance is nullptr, geo_assert will
         // call the Logger to print the assertion failure, thus ending in a
         // infinite loop.
         if(instance_ == nullptr) {
@@ -367,51 +415,60 @@ namespace GEO {
         return instance_;
     }
 
+    std::ostream& Logger::err_console() {
+        static_cast<CERRStream*>(err_console_)->lock();
+        return *err_console_;
+    }
+
     std::ostream& Logger::div(const std::string& title) {
-	std::ostream& result = 
-   	    (is_initialized() && !Process::is_running_threads()) ?
+        std::ostream& result =
+            (is_initialized() && !Process::is_running_threads()) ?
             instance()->div_stream(title) :
-            (std::cerr << "=====" << title << std::endl);
-	return result;
+            (instance()->err_console() << "=====" << title << std::endl);
+        return result;
     }
 
     std::ostream& Logger::out(const std::string& feature) {
-	std::ostream& result =
-	    (is_initialized() && !Process::is_running_threads()) ?
+        std::ostream& result =
+            (is_initialized() && !Process::is_running_threads()) ?
             instance()->out_stream(feature) :
-            (std::cerr << "    [" << feature << "] ");
-	return result;
+            (instance()->err_console() << "    [" << feature << "] ");
+	for(index_t i=0; i<instance_->indent_; ++i) {
+	    result << "| ";
+	}
+        return result;
     }
 
     std::ostream& Logger::err(const std::string& feature) {
-	std::ostream& result = 
-	    (is_initialized() && !Process::is_running_threads()) ?	    
+        std::ostream& result =
+            (is_initialized() && !Process::is_running_threads()) ?
             instance()->err_stream(feature) :
-            (std::cerr << "(E)-[" << feature << "] ");
-	return result;
+            (instance()->err_console() << "(E)-[" << feature << "] ");
+        return result;
     }
 
     std::ostream& Logger::warn(const std::string& feature) {
-	std::ostream& result = 
-	    (is_initialized() && !Process::is_running_threads()) ?	    	    
+        std::ostream& result =
+            (is_initialized() && !Process::is_running_threads()) ?
             instance()->warn_stream(feature) :
-            (std::cerr << "(W)-[" << feature << "] ");
-	return result;
+            (instance()->err_console() << "(W)-[" << feature << "] ");
+        return result;
     }
 
     std::ostream& Logger::status() {
-	std::ostream& result =	
-	    (is_initialized() && !Process::is_running_threads()) ?	    	    	
+        std::ostream& result =
+            (is_initialized() && !Process::is_running_threads()) ?
             instance()->status_stream() :
-            (std::cerr << "[status] ");
-	return result;
+            (instance()->err_console() << "[status] ");
+        return result;
     }
 
     std::ostream& Logger::div_stream(const std::string& title) {
         if(!quiet_) {
             current_feature_changed_ = true;
             current_feature_.clear();
-            for(auto it : clients_) {
+            LoggerClients clients = clients_; // clients_ may be modified !
+            for(auto it : clients) {
                 it->div(title);
             }
         }
@@ -449,15 +506,16 @@ namespace GEO {
     void Logger::notify_out(const std::string& message) {
         if(
             (log_everything_ &&
-                log_features_exclude_.find(current_feature_) ==
-                log_features_exclude_.end())
+             log_features_exclude_.find(current_feature_) ==
+             log_features_exclude_.end())
             || (log_features_.find(current_feature_) != log_features_.end())
         ) {
             std::string feat_msg =
                 CmdLine::ui_feature(current_feature_, current_feature_changed_)
                 + message;
 
-	    for(auto it : clients_) {
+            LoggerClients clients = clients_; // clients_ may be modified !
+            for(auto it : clients) {
                 it->out(feat_msg);
             }
 
@@ -471,7 +529,8 @@ namespace GEO {
             CmdLine::ui_feature(current_feature_, current_feature_changed_)
             + msg;
 
-        for(auto it : clients_) {
+        LoggerClients clients = clients_; // clients_ may be modified !
+        for(auto it : clients) {
             it->warn(feat_msg);
             it->status(msg);
         }
@@ -485,23 +544,25 @@ namespace GEO {
             CmdLine::ui_feature(current_feature_, current_feature_changed_)
             + msg;
 
-	if(notifying_error_) {
-	    std::cerr << "Error while displaying error (!):"
-		      << feat_msg << std::endl;
-	} else {
-	    notifying_error_ = true;
-	    for(auto it : clients_) {
-		it->err(feat_msg);
-		it->status(msg);
-	    }
-	    notifying_error_ = false;
-	}
+        if(notifying_error_) {
+            std::cerr << "Error while displaying error (!):"
+                      << feat_msg << std::endl;
+        } else {
+            notifying_error_ = true;
+            LoggerClients clients = clients_; // clients_ may be modified !
+            for(auto it : clients) {
+                it->err(feat_msg);
+                it->status(msg);
+            }
+            notifying_error_ = false;
+        }
 
         current_feature_changed_ = false;
     }
 
     void Logger::notify_status(const std::string& message) {
-        for(auto it : clients_) {
+        LoggerClients clients = clients_; // clients_ may be modified !
+        for(auto it : clients) {
             it->status(message);
         }
 
@@ -528,7 +589,7 @@ namespace GEO {
     }
 
     /************************************************************************/
-    
+
 }
 
 extern "C" {
@@ -539,9 +600,9 @@ extern "C" {
 
         va_list args;
 
-        // Get the number of characters to be printed.        
+        // Get the number of characters to be printed.
         va_start(args, format);
-        int nb = vsnprintf(nullptr, 0, format, args)+1; // +1, I don't know why...
+        int nb = vsnprintf(nullptr, 0, format, args)+1; // +1, I don't know why.
         va_end(args);
 
         // Generate the output string
@@ -582,11 +643,11 @@ extern "C" {
                 GEO::Logger::out("") << last_string << lines[i] << std::endl;
                 last_string.clear();
             } else {
-                GEO::Logger::out("") << lines[i] << std::endl;                
+                GEO::Logger::out("") << lines[i] << std::endl;
             }
         }
 
-	return nb;
+        return nb;
     }
 
     int geogram_fprintf(FILE* out, const char* format, ...) {
@@ -596,7 +657,7 @@ extern "C" {
 
         va_list args;
 
-        // Get the number of characters to be printed.        
+        // Get the number of characters to be printed.
         va_start(args, format);
         int nb = vsnprintf(nullptr, 0, format, args)+1; // +1, I don't know why...
         va_end(args);
@@ -639,23 +700,22 @@ extern "C" {
                 if(out == stdout) {
                     GEO::Logger::out("") << last_string << lines[i] << std::endl;
                 } else if(out == stderr) {
-                    GEO::Logger::err("") << last_string << lines[i] << std::endl;                    
+                    GEO::Logger::err("") << last_string << lines[i] << std::endl;
                 } else {
-                    fprintf(out, "%s%s", last_string.c_str(), lines[i]);                    
+                    fprintf(out, "%s%s", last_string.c_str(), lines[i]);
                 }
                 last_string.clear();
             } else {
                 if(out == stdout) {
                     GEO::Logger::out("") << lines[i] << std::endl;
                 } else if(out == stderr) {
-                    GEO::Logger::err("") << lines[i] << std::endl;                    
+                    GEO::Logger::err("") << lines[i] << std::endl;
                 } else {
-                    fprintf(out, "%s", lines[i]);                    
+                    fprintf(out, "%s", lines[i]);
                 }
             }
         }
-	
-	return nb;
+
+        return nb;
     }
 }
-
