@@ -13,7 +13,7 @@
  *  * Neither the name of the ALICE Project-Team nor the names of its
  *  contributors may be used to endorse or promote products derived from this
  *  software without specific prior written permission.
- * 
+ *
  *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -42,6 +42,7 @@
 
 #include <geogram/basic/common.h>
 #include <geogram/basic/vecg.h>
+#include <initializer_list>
 
 /**
  * \file geogram/basic/matrix.h
@@ -71,7 +72,7 @@ namespace GEO {
         typedef FT value_type;
 
         /** The dimension of the matrix */
-        static const index_t dim = DIM;
+        static constexpr index_t dim = DIM;
 
         /**
          * \brief Default constructor
@@ -96,7 +97,26 @@ namespace GEO {
                 }
             }
         }
-        
+
+        /**
+         * \brief Constructs a matrix from 2d array of initializers.
+         * \param[in] Mi a 2d array of values to be copied to the matrix.
+         */
+        Matrix(const std::initializer_list< std::initializer_list<FT> >& Mi) {
+            index_t i = 0;
+            for(auto& it: Mi) {
+                index_t j = 0;
+                for(auto& jt: it) {
+                    geo_debug_assert(i < DIM);
+                    geo_debug_assert(j < DIM);
+                    coeff_[i][j] = jt;
+                    ++j;
+                }
+                ++i;
+            }
+        }
+
+
         /**
          * \brief Gets the matrix dimension
          * \return the value of \p DIM
@@ -146,7 +166,7 @@ namespace GEO {
             }
             return true;
         }
-        
+
         /**
          * \brief Gets a modifiable element
          * \details Gets element at row \p i and column \p j in the matrix. If
@@ -374,7 +394,7 @@ namespace GEO {
                     }
                 }
             }
-            
+
             return true;
         }
 
@@ -549,8 +569,7 @@ namespace GEO {
     }
 
     /************************************************************************/
-    
+
 }
 
 #endif
-
