@@ -1185,8 +1185,8 @@ namespace GEO {
     /**
      * \brief Reads a vector from a stream
      * \details This reads \p DIM coordinates from the input stream \p in and
-     * stores them in vector \p v. Understands both "x y z" and
-     *  "[x, y, z]" formats.
+     * stores them in vector \p v. Understands both "x y z",
+     *  "[x, y, z]" and "{x, y, z}" formats.
      * \param[in] in the input stream
      * \param[out] v the vector to read
      * \return a reference to the input stream \p in
@@ -1200,7 +1200,7 @@ namespace GEO {
         while(isspace(in.peek())) {
             in.get(c);
         }
-        if(in.peek() == '[') {
+        if(in.peek() == '[' || in.peek() == '{') {
             in.get(c);
         }
         while(isspace(in.peek())) {
@@ -1218,7 +1218,7 @@ namespace GEO {
                 in.get(c);
             }
         }
-        if(in.peek() == ']') {
+        if(in.peek() == ']' || in.peek() == '}') {
             in.get(c);
         }
         return in;
