@@ -149,9 +149,7 @@ namespace GEO {
 #endif
     }
 
-    Delaunay* Delaunay::create(
-        coord_index_t dim, const std::string& name_in
-    ) {
+    Delaunay* Delaunay::create(coord_index_t dim, const std::string& name_in) {
 
         std::string name = name_in;
         if(name == "default") {
@@ -209,9 +207,7 @@ namespace GEO {
     Delaunay::~Delaunay() {
     }
 
-    void Delaunay::set_vertices(
-        index_t nb_vertices, const double* vertices
-    ) {
+    void Delaunay::set_vertices(index_t nb_vertices, const double* vertices) {
         nb_vertices_ = nb_vertices;
         vertices_ = vertices;
     }
@@ -223,7 +219,7 @@ namespace GEO {
 
     void Delaunay::set_arrays(
         index_t nb_cells,
-        const signed_index_t* cell_to_v, const signed_index_t* cell_to_cell
+        const index_t* cell_to_v, const index_t* cell_to_cell
     ) {
         nb_cells_ = nb_cells;
         cell_to_v_ = cell_to_v;
@@ -278,27 +274,27 @@ namespace GEO {
     }
 
     void Delaunay::get_neighbors_internal(
-        index_t v, vector<index_t>& neighbors
+	index_t v, vector<index_t>& neighbors
     ) const {
         // Step 1: traverse the incident cells list, and insert
         // all neighbors (may be duplicated)
         neighbors.resize(0);
-        signed_index_t vt = v_to_cell_[v];
-        if(vt != -1) { // Happens when there are duplicated vertices.
-            index_t t = index_t(vt);
+        index_t vt = v_to_cell_[v];
+        if(vt != NO_INDEX) { // Happens when there are duplicated vertices.
+            index_t t = vt;
             do {
-                index_t lvit = index(t, signed_index_t(v));
+                index_t lvit = index(t, v);
                 // In the current cell, test all edges incident
                 // to current vertex 'it'
                 for(index_t lv = 0; lv < cell_size(); lv++) {
                     if(lvit != lv) {
-                        signed_index_t neigh = cell_vertex(t, lv);
-                        geo_debug_assert(neigh != -1);
-                        neighbors.push_back(index_t(neigh));
+                        index_t neigh = cell_vertex(t, lv);
+                        geo_debug_assert(neigh != NO_INDEX);
+                        neighbors.push_back(neigh);
                     }
                 }
-                t = index_t(next_around_vertex(t, index(t, signed_index_t(v))));
-            } while(t != index_t(vt));
+                t = next_around_vertex(t, index(t, v));
+            } while(t != vt);
         }
 
         // Step 2: Sort the neighbors and remove all duplicates
@@ -323,21 +319,21 @@ namespace GEO {
         // tet chaining is at t2v_[nb_vertices].
 
         if(keeps_infinite()) {
-            v_to_cell_.assign(nb_vertices()+1, -1);
+            v_to_cell_.assign(nb_vertices()+1, NO_INDEX);
             for(index_t c = 0; c < nb_cells(); c++) {
                 for(index_t lv = 0; lv < cell_size(); lv++) {
-                    signed_index_t v = cell_vertex(c, lv);
-                    if(v == -1) {
-                        v = signed_index_t(nb_vertices());
+                    index_t v = cell_vertex(c, lv);
+                    if(v == NO_INDEX) {
+                        v = nb_vertices();
                     }
-                    v_to_cell_[v] = signed_index_t(c);
+                    v_to_cell_[v] = c;
                 }
             }
         } else {
-            v_to_cell_.assign(nb_vertices(), -1);
+            v_to_cell_.assign(nb_vertices(), NO_INDEX);
             for(index_t c = 0; c < nb_cells(); c++) {
                 for(index_t lv = 0; lv < cell_size(); lv++) {
-                    v_to_cell_[cell_vertex(c, lv)] = signed_index_t(c);
+                    v_to_cell_[cell_vertex(c, lv)] = c;
                 }
             }
         }
@@ -350,10 +346,10 @@ namespace GEO {
         cicl_.resize(cell_size() * nb_cells());
 
         for(index_t v = 0; v < nb_vertices(); ++v) {
-            signed_index_t t = v_to_cell_[v];
-            if(t != -1) {
-                index_t lv = index(index_t(t), signed_index_t(v));
-                set_next_around_vertex(index_t(t), lv, index_t(t));
+            index_t t = v_to_cell_[v];
+            if(t != NO_INDEX) {
+                index_t lv = index(t, v);
+                set_next_around_vertex(t, lv, t);
             }
         }
 
@@ -361,21 +357,21 @@ namespace GEO {
 
             {
                 // Process the infinite vertex at index nb_vertices().
-                signed_index_t t = v_to_cell_[nb_vertices()];
-                if(t != -1) {
-                    index_t lv = index(index_t(t), -1);
-                    set_next_around_vertex(index_t(t), lv, index_t(t));
+                index_t t = v_to_cell_[nb_vertices()];
+                if(t != NO_INDEX) {
+                    index_t lv = index(t, NO_INDEX);
+                    set_next_around_vertex(t, lv, t);
                 }
             }
 
             for(index_t t = 0; t < nb_cells(); ++t) {
                 for(index_t lv = 0; lv < cell_size(); ++lv) {
-                    signed_index_t v = cell_vertex(t, lv);
-                    index_t vv = (v == -1) ? nb_vertices() : index_t(v);
-                    if(v_to_cell_[vv] != signed_index_t(t)) {
-                        index_t t1 = index_t(v_to_cell_[vv]);
-                        index_t lv1 = index(t1, signed_index_t(v));
-                        index_t t2 = index_t(next_around_vertex(t1, lv1));
+                    index_t v = cell_vertex(t, lv);
+                    index_t vv = (v == NO_INDEX) ? nb_vertices() : v;
+                    if(v_to_cell_[vv] != t) {
+                        index_t t1 = v_to_cell_[vv];
+                        index_t lv1 = index(t1, v);
+                        index_t t2 = next_around_vertex(t1, lv1);
                         set_next_around_vertex(t1, lv1, t);
                         set_next_around_vertex(t, lv, t2);
                     }
@@ -386,11 +382,11 @@ namespace GEO {
         } else {
             for(index_t t = 0; t < nb_cells(); ++t) {
                 for(index_t lv = 0; lv < cell_size(); ++lv) {
-                    index_t v = index_t(cell_vertex(t, lv));
-                    if(v_to_cell_[v] != signed_index_t(t)) {
-                        index_t t1 = index_t(v_to_cell_[v]);
-                        index_t lv1 = index(t1, signed_index_t(v));
-                        index_t t2 = index_t(next_around_vertex(t1, lv1));
+                    index_t v = cell_vertex(t, lv);
+                    if(v_to_cell_[v] != t) {
+                        index_t t1 = v_to_cell_[v];
+                        index_t lv1 = index(t1, v);
+                        index_t t2 = next_around_vertex(t1, lv1);
                         set_next_around_vertex(t1, lv1, t);
                         set_next_around_vertex(t, lv, t2);
                     }
@@ -418,7 +414,7 @@ namespace GEO {
     bool Delaunay::cell_is_infinite(index_t c) const {
         geo_debug_assert(c < nb_cells());
         for(index_t lv=0; lv < cell_size(); ++lv) {
-            if(cell_vertex(c,lv) == -1) {
+            if(cell_vertex(c,lv) == NO_INDEX) {
                 return true;
             }
         }
@@ -428,6 +424,6 @@ namespace GEO {
     index_t Delaunay::region(index_t t) const {
         geo_argused(t);
         geo_debug_assert(t < nb_cells());
-        return index_t(-1);
+        return NO_INDEX;
     }
 }
