@@ -71,8 +71,7 @@ namespace {
      *   Shewchuk's version.
      */
     inline Sign orient_2d_inexact(
-        const double* p0, const double* p1,
-        const double* p2
+	const double* p0, const double* p1, const double* p2
     ) {
         double a11 = p1[0] - p0[0] ;
         double a12 = p1[1] - p0[1] ;
@@ -167,14 +166,10 @@ namespace GEO {
     Delaunay2d::~Delaunay2d() {
     }
 
-    void Delaunay2d::set_vertices(
-        index_t nb_vertices, const double* vertices
-    ) {
+    void Delaunay2d::set_vertices(index_t nb_vertices, const double* vertices) {
         has_empty_cells_ = false;
-        Stopwatch* W = nullptr;
-        if(benchmark_mode_) {
-            W = new Stopwatch("DelInternal");
-        }
+	Stopwatch W("DelInternal", benchmark_mode_);
+
         cur_stamp_ = 0;
         if(weighted_) {
             heights_.resize(nb_vertices);
@@ -221,7 +216,7 @@ namespace GEO {
 
         double sorting_time = 0;
         if(benchmark_mode_) {
-            sorting_time = W->elapsed_time();
+            sorting_time = W.elapsed_time();
             Logger::out("DelInternal1") << "BRIO sorting:"
                                         << sorting_time
                                         << std::endl;
@@ -255,10 +250,9 @@ namespace GEO {
 
         if(benchmark_mode_) {
             Logger::out("DelInternal2") << "Core insertion algo:"
-                                        << W->elapsed_time() - sorting_time
+                                        << W.elapsed_time() - sorting_time
                                         << std::endl;
         }
-        delete W;
 
         if(debug_mode_) {
             check_combinatorics(verbose_debug_mode_);
@@ -290,27 +284,27 @@ namespace GEO {
                         Memory::copy(
                             &cell_to_v_store_[nb_triangles * 3],
                             &cell_to_v_store_[t * 3],
-                            3 * sizeof(signed_index_t)
+                            3 * sizeof(index_t)
                         );
                         Memory::copy(
                             &cell_to_cell_store_[nb_triangles * 3],
                             &cell_to_cell_store_[t * 3],
-                            3 * sizeof(signed_index_t)
+                            3 * sizeof(index_t)
                         );
                     }
                     old2new[t] = nb_triangles;
                     ++nb_triangles;
                 } else {
-                    old2new[t] = index_t(-1);
+                    old2new[t] = NO_INDEX;
                     ++nb_triangles_to_delete;
                 }
             }
             cell_to_v_store_.resize(3 * nb_triangles);
             cell_to_cell_store_.resize(3 * nb_triangles);
             for(index_t i = 0; i < 3 * nb_triangles; ++i) {
-                signed_index_t t = cell_to_cell_store_[i];
-                geo_debug_assert(t >= 0);
-                t = signed_index_t(old2new[t]);
+                index_t t = cell_to_cell_store_[i];
+                geo_debug_assert(t != NO_INDEX);
+                t = old2new[t];
                 // Note: t can be equal to -1 when a real trgl is
                 // adjacent to a virtual one (and this is how the
                 // rest of Vorpaline expects to see trgls on the
@@ -359,10 +353,10 @@ namespace GEO {
                 --infinite_ptr;
             }
             for(index_t i = 0; i < 3 * nb_triangles; ++i) {
-                signed_index_t t = cell_to_cell_store_[i];
-                geo_debug_assert(t >= 0);
-                t = signed_index_t(old2new[t]);
-                geo_debug_assert(t >= 0);
+                index_t t = cell_to_cell_store_[i];
+                geo_debug_assert(t != NO_INDEX);
+                t = old2new[t];
+                geo_debug_assert(t != NO_INDEX);
                 cell_to_cell_store_[i] = t;
             }
         }
@@ -387,7 +381,7 @@ namespace GEO {
         // Not mandatory, but doing so makes it possible to
         // use locate() in derived classes outside of
         // set_vertices().
-        cell_next_.assign(cell_next_.size(),~index_t(0));
+        cell_next_.assign(cell_next_.size(),NO_INDEX);
     }
 
     index_t Delaunay2d::nearest_vertex(const double* p) const {
@@ -415,23 +409,21 @@ namespace GEO {
 
         // Find the nearest vertex among t's vertices
         for(index_t lv = 0; lv < 3; ++lv) {
-            signed_index_t v = triangle_vertex(t, lv);
+            index_t v = triangle_vertex(t, lv);
             // If the triangle is virtual, then the first vertex
             // is the vertex at infinity and is skipped.
-            if(v < 0) {
+            if(v == NO_INDEX) {
                 continue;
             }
-            double cur_sq_dist = Geom::distance2(p, vertex_ptr(index_t(v)), 2);
+            double cur_sq_dist = Geom::distance2(p, vertex_ptr(v), 2);
             if(cur_sq_dist < sq_dist) {
                 sq_dist = cur_sq_dist;
-                result = index_t(v);
+                result = v;
             }
         }
         return result;
     }
 
-
-
     index_t Delaunay2d::locate_inexact(
         const double* p, index_t hint, index_t max_iter
     ) const {
@@ -453,7 +445,7 @@ namespace GEO {
         if(triangle_is_virtual(hint)) {
             for(index_t lf = 0; lf < 3; ++lf) {
                 if(triangle_vertex(hint, lf) == VERTEX_AT_INFINITY) {
-                    hint = index_t(triangle_adjacent(hint, lf));
+                    hint = triangle_adjacent(hint, lf);
                     geo_debug_assert(hint != NO_TRIANGLE);
                     break;
                 }
@@ -472,18 +464,16 @@ namespace GEO {
 
             for(index_t le = 0; le < 3; ++le) {
 
-                signed_index_t s_t_next = triangle_adjacent(t,le);
+                index_t t_next = triangle_adjacent(t,le);
 
                 //  If the opposite trgl is -1, then it means that
                 // we are trying to locate() (e.g. called from
                 // nearest_vertex) within a triangulation
                 // from which the infinite trgls were removed.
-                if(s_t_next == -1) {
+                if(t_next == NO_INDEX) {
                     return NO_TRIANGLE;
                 }
 
-                index_t t_next = index_t(s_t_next);
-
                 //   If the candidate next triangle is the
                 // one we came from, then we know already that
                 // the orientation is positive, thus we examine
@@ -537,8 +527,7 @@ namespace GEO {
 
 
     index_t Delaunay2d::locate(
-        const double* p, index_t hint, bool thread_safe,
-        Sign* orient
+        const double* p, index_t hint, bool thread_safe, Sign* orient
     ) const {
 
         //   Try improving the hint by using the
@@ -580,7 +569,7 @@ namespace GEO {
         if(triangle_is_virtual(hint)) {
             for(index_t le = 0; le < 3; ++le) {
                 if(triangle_vertex(hint, le) == VERTEX_AT_INFINITY) {
-                    hint = index_t(triangle_adjacent(hint, le));
+                    hint = triangle_adjacent(hint, le);
                     geo_debug_assert(hint != NO_TRIANGLE);
                     break;
                 }
@@ -611,25 +600,22 @@ namespace GEO {
             for(index_t de = 0; de < 3; ++de) {
                 index_t le = (e0 + de) % 3;
 
-                signed_index_t s_t_next = triangle_adjacent(t,le);
+                index_t t_next = triangle_adjacent(t,le);
 
                 //  If the opposite triangle is -1, then it means that
                 // we are trying to locate() (e.g. called from
                 // nearest_vertex) within a triangulation
                 // from which the infinite trgls were removed.
-                if(s_t_next == -1) {
+                if(t_next == NO_INDEX) {
                     if(thread_safe) {
                         Process::release_spinlock(lock);
                     }
                     return NO_TRIANGLE;
                 }
 
-                index_t t_next = index_t(s_t_next);
-
                 geo_debug_assert(!triangle_is_free(t_next));
                 geo_debug_assert(!triangle_is_in_list(t_next));
 
-
                 //   If the candidate next triangle is the
                 // one we came from, then we know already that
                 // the orientation is positive, thus we examine
@@ -747,13 +733,13 @@ namespace GEO {
         if(!weighted_ && nb_zero != 0) {
             for(index_t le = 0; le < 3; ++le) {
                 if(orient[le] == ZERO) {
-                    index_t t2 = index_t(triangle_adjacent(t, le));
+                    index_t t2 = triangle_adjacent(t, le);
                     add_triangle_to_list(t2, first, last);
                 }
             }
             for(index_t le = 0; le < 3; ++le) {
                 if(orient[le] == ZERO) {
-                    index_t t2 = index_t(triangle_adjacent(t, le));
+                    index_t t2 = triangle_adjacent(t, le);
                     find_conflict_zone_iterative(
                         p,t2,t_bndry,e_bndry,first,last
                     );
@@ -779,7 +765,7 @@ namespace GEO {
             S_.pop();
 
             for(index_t le = 0; le < 3; ++le) {
-                index_t t2 = index_t(triangle_adjacent(t, le));
+                index_t t2 = triangle_adjacent(t, le);
 
                 if(
                     triangle_is_in_list(t2) || // known as conflict
@@ -812,24 +798,24 @@ namespace GEO {
 
         index_t t = t1;
         index_t e = t1ebord;
-        index_t t_adj = index_t(triangle_adjacent(t,e));
+        index_t t_adj = triangle_adjacent(t,e);
 
-        geo_debug_assert(t_adj != index_t(-1));
+        geo_debug_assert(t_adj != NO_INDEX);
 
         geo_debug_assert(triangle_is_in_list(t));
         geo_debug_assert(!triangle_is_in_list(t_adj));
 
 
-        index_t new_t_first = index_t(-1);
-        index_t new_t_prev  = index_t(-1);
+        index_t new_t_first = NO_INDEX;
+        index_t new_t_prev  = NO_INDEX;
 
         do {
 
-            signed_index_t v1 = triangle_vertex(t, (e+1)%3);
-            signed_index_t v2 = triangle_vertex(t, (e+2)%3);
+            index_t v1 = triangle_vertex(t, (e+1)%3);
+            index_t v2 = triangle_vertex(t, (e+2)%3);
 
             // Create new triangle
-            index_t new_t = new_triangle(signed_index_t(v_in), v1, v2);
+            index_t new_t = new_triangle(v_in, v1, v2);
 
             //   Connect new triangle to triangle on the other
             // side of the conflict zone.
@@ -840,15 +826,15 @@ namespace GEO {
 
             // Move to next triangle
             e = (e + 1)%3;
-            t_adj = index_t(triangle_adjacent(t,e));
+            t_adj = triangle_adjacent(t,e);
             while(triangle_is_in_list(t_adj)) {
                 t = t_adj;
                 e = (find_triangle_vertex(t,v2) + 2)%3;
-                t_adj = index_t(triangle_adjacent(t,e));
-                geo_debug_assert(t_adj != index_t(-1));
+                t_adj = triangle_adjacent(t,e);
+                geo_debug_assert(t_adj != NO_INDEX);
             }
 
-            if(new_t_prev == index_t(-1)) {
+            if(new_t_prev == NO_INDEX) {
                 new_t_first = new_t;
             } else {
                 set_triangle_adjacent(new_t_prev, 1, new_t);
@@ -868,7 +854,7 @@ namespace GEO {
 
     index_t Delaunay2d::insert(index_t v, index_t hint) {
         index_t t_bndry = NO_TRIANGLE;
-        index_t e_bndry = index_t(-1);
+        index_t e_bndry = NO_INDEX;
         index_t first_conflict = NO_TRIANGLE;
         index_t last_conflict  = NO_TRIANGLE;
 
@@ -923,7 +909,9 @@ namespace GEO {
         Sign s = ZERO;
         while(
             iv2 < nb_vertices() &&
-            (s = PCK::orient_2d(vertex_ptr(iv0), vertex_ptr(iv1), vertex_ptr(iv2))) == ZERO
+            (s = PCK::orient_2d(
+		vertex_ptr(iv0), vertex_ptr(iv1), vertex_ptr(iv2)
+	    )) == ZERO
         ) {
             ++iv2;
         }
@@ -935,18 +923,14 @@ namespace GEO {
         }
 
         // Create the first triangle
-        index_t t0 = new_triangle(
-            signed_index_t(iv0),
-            signed_index_t(iv1),
-            signed_index_t(iv2)
-        );
+        index_t t0 = new_triangle(iv0, iv1, iv2);
 
         // Create the first three virtual triangles surrounding it
         index_t t[3];
         for(index_t e = 0; e < 3; ++e) {
             // In reverse order since it is an adjacent triangle
-            signed_index_t v1 = triangle_vertex(t0, triangle_edge_vertex(e,1));
-            signed_index_t v2 = triangle_vertex(t0, triangle_edge_vertex(e,0));
+            index_t v1 = triangle_vertex(t0, triangle_edge_vertex(e,1));
+            index_t v2 = triangle_vertex(t0, triangle_edge_vertex(e,0));
             t[e] = new_triangle(VERTEX_AT_INFINITY, v1, v2);
         }
 
@@ -999,16 +983,16 @@ namespace GEO {
     }
 
     void Delaunay2d::show_triangle_adjacent(index_t t, index_t le) const {
-        signed_index_t adj = triangle_adjacent(t, le);
-        if(adj != -1) {
-            std::cerr << (triangle_is_in_list(index_t(adj)) ? '*' : ' ');
+        index_t adj = triangle_adjacent(t, le);
+        if(adj != NO_INDEX) {
+            std::cerr << (triangle_is_in_list(adj) ? '*' : ' ');
         }
         std::cerr << adj;
         std::cerr << ' ';
     }
 
     void Delaunay2d::show_list(
-        index_t first, const std::string& list_name
+	index_t first, const std::string& list_name
     ) const {
         index_t t = first;
         std::cerr << "tri list: " << list_name << std::endl;
@@ -1041,19 +1025,19 @@ namespace GEO {
   }
 */
                 for(index_t le = 0; le < 3; ++le) {
-                    if(triangle_adjacent(t, le) == -1) {
+                    if(triangle_adjacent(t, le) == NO_INDEX) {
                         std::cerr << le << ":Missing adjacent tri"
                                   << std::endl;
                         ok = false;
-                    } else if(triangle_adjacent(t, le) == signed_index_t(t)) {
+                    } else if(triangle_adjacent(t, le) == t) {
                         std::cerr << le << ":Tri is adjacent to itself"
                                   << std::endl;
                         ok = false;
                     } else {
-                        index_t t2 = index_t(triangle_adjacent(t, le));
+                        index_t t2 = triangle_adjacent(t, le);
                         bool found = false;
                         for(index_t le2 = 0; le2 < 3; ++le2) {
-                            if(triangle_adjacent(t2, le2) == signed_index_t(t)) {
+                            if(triangle_adjacent(t2, le2) == t) {
                                 found = true;
                             }
                         }
@@ -1067,7 +1051,7 @@ namespace GEO {
                 }
                 index_t nb_infinite = 0;
                 for(index_t lv = 0; lv < 3; ++lv) {
-                    if(triangle_vertex(t, lv) == -1) {
+                    if(triangle_vertex(t, lv) == NO_INDEX) {
                         ++nb_infinite;
                     }
                 }
@@ -1078,9 +1062,9 @@ namespace GEO {
                 }
             }
             for(index_t lv = 0; lv < 3; ++lv) {
-                signed_index_t v = triangle_vertex(t, lv);
-                if(v >= 0) {
-                    v_has_triangle[index_t(v)] = true;
+                index_t v = triangle_vertex(t, lv);
+                if(v != NO_INDEX) {
+                    v_has_triangle[v] = true;
                 }
             }
         }
@@ -1103,12 +1087,11 @@ namespace GEO {
         bool ok = true;
         for(index_t t = 0; t < max_t(); ++t) {
             if(!triangle_is_free(t)) {
-                signed_index_t v0 = triangle_vertex(t, 0);
-                signed_index_t v1 = triangle_vertex(t, 1);
-                signed_index_t v2 = triangle_vertex(t, 2);
+                index_t v0 = triangle_vertex(t, 0);
+                index_t v1 = triangle_vertex(t, 1);
+                index_t v2 = triangle_vertex(t, 2);
                 for(index_t v = 0; v < nb_vertices(); ++v) {
-                    signed_index_t sv = signed_index_t(v);
-                    if(sv == v0 || sv == v1 || sv == v2) {
+                    if(v == v0 || v == v1 || v == v2) {
                         continue;
                     }
                     if(triangle_is_conflict(t, vertex_ptr(v))) {
