@@ -64,36 +64,18 @@ namespace {
         vec3& nearest_p,
         double& squared_dist
     ) {
-        if(M.facets.nb_vertices(f) == 3) {
-            index_t c = M.facets.corners_begin(f);
-            const vec3& p1 = Geom::mesh_vertex(M, M.facet_corners.vertex(c));
-            const vec3& p2 = Geom::mesh_vertex(M, M.facet_corners.vertex(c+1));
-            const vec3& p3 = Geom::mesh_vertex(M, M.facet_corners.vertex(c+2));
-            double lambda1, lambda2, lambda3;  // barycentric coords, not used.
-            squared_dist = Geom::point_triangle_squared_distance(
-                p, p1, p2, p3, nearest_p, lambda1, lambda2, lambda3
-            );
-        } else {
-            squared_dist = Numeric::max_float64();
-            index_t c1 = M.facets.corners_begin(f);
-            const vec3& p1 = Geom::mesh_vertex(M, M.facet_corners.vertex(c1));
-            for(index_t c2 = c1+1; c2+1<M.facets.corners_end(f); ++c2) {
-                const vec3& p2 =
-                    Geom::mesh_vertex(M, M.facet_corners.vertex(c2));
-                index_t c3 = c2+1;
-                const vec3& p3 =
-                    Geom::mesh_vertex(M, M.facet_corners.vertex(c3));
-                double lambda1, lambda2, lambda3;  // barycentric coords,unused.
-                vec3 cur_nearest_p;
-                double cur_squared_dist = Geom::point_triangle_squared_distance(
-                    p, p1, p2, p3, cur_nearest_p, lambda1, lambda2, lambda3
-                );
-                if(cur_squared_dist < squared_dist) {
-                    squared_dist = cur_squared_dist;
-                    nearest_p = cur_nearest_p;
-                }
-            }
-        }
+	squared_dist = Numeric::max_float64();
+	for(auto [ p1, p2, p3] : M.facets.triangle_points(f)) {
+	    double lambda1, lambda2, lambda3;  // barycentric coords,unused.
+	    vec3 cur_nearest_p;
+	    double cur_squared_dist = Geom::point_triangle_squared_distance(
+		p, p1, p2, p3, cur_nearest_p, lambda1, lambda2, lambda3
+	    );
+	    if(cur_squared_dist < squared_dist) {
+		squared_dist = cur_squared_dist;
+		nearest_p = cur_nearest_p;
+	    }
+	}
     }
 
     /**
@@ -175,10 +157,10 @@ namespace {
     bool mesh_tet_contains_point(
         const Mesh& M, index_t t, const vec3& p
     ) {
-        const vec3& p0 = Geom::mesh_vertex(M, M.cells.vertex(t,0));
-        const vec3& p1 = Geom::mesh_vertex(M, M.cells.vertex(t,1));
-        const vec3& p2 = Geom::mesh_vertex(M, M.cells.vertex(t,2));
-        const vec3& p3 = Geom::mesh_vertex(M, M.cells.vertex(t,3));
+        const vec3& p0 = M.cells.point(t,0);
+        const vec3& p1 = M.cells.point(t,1);
+        const vec3& p2 = M.cells.point(t,2);
+        const vec3& p3 = M.cells.point(t,3);
 
         Sign s[4];
         s[0] = PCK::orient_3d(p, p1, p2, p3);
@@ -208,9 +190,9 @@ namespace {
         index_t i = M.facets.vertex(t,0);
         index_t j = M.facets.vertex(t,1);
         index_t k = M.facets.vertex(t,2);
-        vec2 p0(M.vertices.point_ptr(i));
-        vec2 p1(M.vertices.point_ptr(j));
-        vec2 p2(M.vertices.point_ptr(k));
+        vec2 p0 = M.vertices.point<2>(i);
+        vec2 p1 = M.vertices.point<2>(j);
+        vec2 p2 = M.vertices.point<2>(k);
 
         Sign s[3];
         s[0] = PCK::orient_2d(p,  p1, p2);
@@ -400,46 +382,38 @@ namespace {
 
 namespace GEO {
 
-    MeshFacetsAABB::MeshFacetsAABB() {
-    }
-
-    MeshFacetsAABB::MeshFacetsAABB(
-        Mesh& M, bool reorder
-    ) {
-        initialize(M, reorder);
-    }
-
-    void MeshFacetsAABB::initialize(
-        Mesh& M, bool reorder
-    ) {
+    void MeshFacetsAABB::initialize(Mesh& M, AABBReorderMode reorder_mode) {
         mesh_ = &M;
         if(mesh_->facets.nb() == 0) {
             return;
         }
-        if(reorder) {
+	switch(reorder_mode) {
+	case AABB_NOREORDER:
+	    break;
+	case AABB_INPLACE:
+	    reorder_.clear();
             mesh_reorder(*mesh_, MESH_ORDER_MORTON);
-        }
+	    break;
+	case AABB_INDIRECT:
+	    compute_mesh_elements_spatial_order(
+		*mesh_, MESH_FACETS, reorder_, MESH_ORDER_MORTON
+	    );
+	    break;
+	}
         AABB::initialize(
             mesh_->facets.nb(),
             [this](Box& B, index_t f) {
                 // Get facet bbox
-                index_t c = mesh_->facets.corners_begin(f);
-                const double* p = mesh_->vertices.point_ptr(
-                    mesh_->facet_corners.vertex(c)
-                );
                 for(coord_index_t coord = 0; coord < 3; ++coord) {
-                    B.xyz_min[coord] = p[coord];
-                    B.xyz_max[coord] = p[coord];
+                    B.xyz_min[coord] = Numeric::max_float64();
+                    B.xyz_max[coord] = -Numeric::max_float64();
                 }
-                for(++c; c < mesh_->facets.corners_end(f); ++c) {
-                    p = mesh_->vertices.point_ptr(
-                        mesh_->facet_corners.vertex(c)
-                    );
+		for(const vec3& p: mesh_->facets.points(f)) {
                     for(coord_index_t coord = 0; coord < 3; ++coord) {
                         B.xyz_min[coord] = std::min(B.xyz_min[coord], p[coord]);
                         B.xyz_max[coord] = std::max(B.xyz_max[coord], p[coord]);
                     }
-                }
+		}
             }
         );
     }
@@ -472,12 +446,10 @@ namespace GEO {
                 n = childr;
             }
         }
-        nearest_f = b;
+        nearest_f = element_in_leaf(b);
 
-        index_t v = mesh_->facet_corners.vertex(
-            mesh_->facets.corners_begin(nearest_f)
-        );
-        nearest_point = Geom::mesh_vertex(*mesh_, v);
+	index_t v = mesh_->facets.vertex(nearest_f, 0);
+        nearest_point = mesh_->vertices.point(v);
         sq_dist = Geom::distance2(p, nearest_point);
     }
 
@@ -491,13 +463,14 @@ namespace GEO {
         // If node is a leaf: compute point-facet distance
         // and replace current if nearer
         if(b + 1 == e) {
+	    index_t f = element_in_leaf(b);
             vec3 cur_nearest_point;
             double cur_sq_dist;
             get_point_facet_nearest_point(
-                *mesh_, p, b, cur_nearest_point, cur_sq_dist
+                *mesh_, p, f, cur_nearest_point, cur_sq_dist
             );
             if(cur_sq_dist < sq_dist) {
-                nearest_f = b;
+                nearest_f = f;
                 nearest_point = cur_nearest_point;
                 sq_dist = cur_sq_dist;
             }
@@ -545,6 +518,79 @@ namespace GEO {
         }
     }
 
+
+    void MeshFacetsAABB::nearest_facet_recursive_filtered(
+        const vec3& p,
+        index_t& nearest_f, vec3& nearest_point, double& sq_dist,
+        index_t n, index_t b, index_t e,
+	std::function<bool(index_t)> filter
+    ) const {
+        geo_debug_assert(e > b);
+
+        // If node is a leaf: compute point-facet distance
+        // and replace current if nearer
+        if(b + 1 == e) {
+	    index_t f = element_in_leaf(b);
+	    if(filter(f)) {
+		vec3 cur_nearest_point;
+		double cur_sq_dist;
+		get_point_facet_nearest_point(
+		    *mesh_, p, f, cur_nearest_point, cur_sq_dist
+		);
+		if(cur_sq_dist < sq_dist) {
+		    nearest_f = f;
+		    nearest_point = cur_nearest_point;
+		    sq_dist = cur_sq_dist;
+		}
+	    }
+            return;
+        }
+        index_t m = b + (e - b) / 2;
+        index_t childl = 2 * n;
+        index_t childr = 2 * n + 1;
+
+        double dl = point_box_signed_squared_distance(p, bboxes_[childl]);
+        double dr = point_box_signed_squared_distance(p, bboxes_[childr]);
+
+        // Traverse the "nearest" child first, so that it has more chances
+        // to prune the traversal of the other child.
+        if(dl < dr) {
+            if(dl < sq_dist) {
+                nearest_facet_recursive_filtered(
+                    p,
+                    nearest_f, nearest_point, sq_dist,
+                    childl, b, m,
+		    filter
+                );
+            }
+            if(dr < sq_dist) {
+                nearest_facet_recursive_filtered(
+                    p,
+                    nearest_f, nearest_point, sq_dist,
+                    childr, m, e,
+		    filter
+                );
+            }
+        } else {
+            if(dr < sq_dist) {
+                nearest_facet_recursive_filtered(
+                    p,
+                    nearest_f, nearest_point, sq_dist,
+                    childr, m, e,
+		    filter
+                );
+            }
+            if(dl < sq_dist) {
+                nearest_facet_recursive_filtered(
+                    p,
+                    nearest_f, nearest_point, sq_dist,
+                    childl, b, m,
+		    filter
+                );
+            }
+        }
+    }
+
     bool MeshFacetsAABB::ray_intersection(
         const Ray& R, double tmax, index_t ignore_f
     ) const {
@@ -600,22 +646,11 @@ namespace GEO {
             return false;
         }
         if(b + 1 == e) {
-            index_t f = b;
+            index_t f = element_in_leaf(b);
             if(f == ignore_f) {
                 return false;
             }
-            index_t c = mesh_->facets.corners_begin(f);
-            const vec3& p1 = Geom::mesh_vertex(
-                *mesh_, mesh_->facet_corners.vertex(c)
-            );
-            ++c;
-            while(c+1 != mesh_->facets.corners_end(f)) {
-                const vec3& p2 = Geom::mesh_vertex(
-                    *mesh_, mesh_->facet_corners.vertex(c)
-                );
-                const vec3& p3 = Geom::mesh_vertex(
-                    *mesh_, mesh_->facet_corners.vertex(c+1)
-                );
+	    for(auto [ p1, p2, p3] : mesh_->facets.triangle_points(f)) {
                 vec3 N;
                 double t,u,v;
                 if(
@@ -625,8 +660,7 @@ namespace GEO {
                 ) {
                     return true;
                 }
-                ++c;
-            }
+	    }
             return false;
         }
         index_t m = b + (e - b) / 2;
@@ -646,25 +680,21 @@ namespace GEO {
             return;
         }
         if(b + 1 == e) {
-            index_t f = b;
+            index_t f = element_in_leaf(b);
             if(f == ignore_f) {
                 return;
             }
-            index_t c = mesh_->facets.corners_begin(f);
-            index_t v1 = mesh_->facet_corners.vertex(c);
-            const vec3& p1 = Geom::mesh_vertex(*mesh_, v1);
-            ++c;
-            while(c+1 != mesh_->facets.corners_end(f)) {
-                index_t v2 = mesh_->facet_corners.vertex(c);
-                index_t v3 = mesh_->facet_corners.vertex(c+1);
-                const vec3& p2 = Geom::mesh_vertex(*mesh_, v2);
-                const vec3& p3 = Geom::mesh_vertex(*mesh_, v3);
+	    for( auto [ v1, v2, v3] : mesh_->facets.triangles(f)) {
+		const vec3& p1 = mesh_->vertices.point(v1);
+		const vec3& p2 = mesh_->vertices.point(v2);
+		const vec3& p3 = mesh_->vertices.point(v3);
                 vec3 N;
                 double t,u,v;
-                if(ray_triangle_intersection(
-                       R.origin,R.direction,p1,p2,p3,t,u,v,N
-                   ) && t<I.t
-                  ) {
+                if(
+		    ray_triangle_intersection(
+			R.origin,R.direction,p1,p2,p3,t,u,v,N
+		    ) && t<I.t
+                ) {
                     I.t = t;
                     I.u = u;
                     I.v = v;
@@ -674,8 +704,7 @@ namespace GEO {
                     I.k = v3;
                     I.f = f;
                 }
-                ++c;
-            }
+	    }
             return;
         }
         index_t m = b + (e - b) / 2;
@@ -709,112 +738,71 @@ namespace GEO {
             return;
         }
         if(b + 1 == e) {
-            index_t f = b;
-            index_t c = mesh_->facets.corners_begin(f);
-            index_t v1 = mesh_->facet_corners.vertex(c);
-            const vec3& p1 = Geom::mesh_vertex(*mesh_, v1);
-            ++c;
-            while(c+1 != mesh_->facets.corners_end(f)) {
-                index_t v2 = mesh_->facet_corners.vertex(c);
-                index_t v3 = mesh_->facet_corners.vertex(c+1);
-                const vec3& p2 = Geom::mesh_vertex(*mesh_, v2);
-                const vec3& p3 = Geom::mesh_vertex(*mesh_, v3);
+            index_t f = element_in_leaf(b);
+	    for( auto [ v1, v2, v3] : mesh_->facets.triangles(f)) {
+		const vec3& p1 = mesh_->vertices.point(v1);
+		const vec3& p2 = mesh_->vertices.point(v2);
+		const vec3& p3 = mesh_->vertices.point(v3);
                 vec3 N;
-                if(ray_triangle_intersection(
-                       R.origin,R.direction,p1,p2,p3,I.t,I.u,I.v,I.N
-                   )
-                  ) {
+                if(
+		    ray_triangle_intersection(
+			R.origin,R.direction,p1,p2,p3,I.t,I.u,I.v,I.N
+		    )
+		) {
                     I.i = v1;
                     I.j = v2;
                     I.k = v3;
                     I.f = f;
                     action(I);
                 }
-                ++c;
-            }
+	    }
             return;
         }
         index_t m = b + (e - b) / 2;
         index_t childl = 2 * n;
         index_t childr = 2 * n + 1;
-        ray_all_intersections_recursive(
-            R, dirinv, action, childr, m, e
-        );
-        ray_all_intersections_recursive(
-            R, dirinv, action, childl, b, m
-        );
+        ray_all_intersections_recursive(R, dirinv, action, childr, m, e);
+        ray_all_intersections_recursive(R, dirinv, action, childl, b, m);
     }
 
 
 /****************************************************************************/
 
-    MeshCellsAABB::MeshCellsAABB() {
-    }
-
-    MeshCellsAABB::MeshCellsAABB(Mesh& M, bool reorder) {
-        initialize(M, reorder);
-    }
-
-    void MeshCellsAABB::initialize(Mesh& M, bool reorder) {
+    void MeshCellsAABB::initialize(Mesh& M, AABBReorderMode reorder_mode) {
         mesh_ = &M;
-        if(reorder) {
+	switch(reorder_mode) {
+	case AABB_NOREORDER:
+	    break;
+	case AABB_INPLACE:
+	    reorder_.clear();
             mesh_reorder(*mesh_, MESH_ORDER_MORTON);
-        }
-        if(mesh_->cells.are_simplices()) {
-            AABB::initialize(
-                mesh_->cells.nb(),
-                [this](Box& B, index_t t) {
-                    // Get tet bbox
-                    const double* p = mesh_->vertices.point_ptr(
-                        mesh_->cells.vertex(t,0)
-                    );
-                    for(coord_index_t coord = 0; coord < 3; ++coord) {
-                        B.xyz_min[coord] = p[coord];
-                        B.xyz_max[coord] = p[coord];
-                    }
-                    for(index_t lv=1; lv<4; ++lv) {
-                        p = mesh_->vertices.point_ptr(
-                            mesh_->cells.vertex(t,lv)
-                        );
-                        for(coord_index_t coord = 0; coord < 3; ++coord) {
-                            B.xyz_min[coord] = std::min(
-                                B.xyz_min[coord], p[coord]
-                            );
-                            B.xyz_max[coord] = std::max(
-                                B.xyz_max[coord], p[coord]
-                            );
-                        }
-                    }
-                }
-            );
-        } else {
-            AABB::initialize(
-                mesh_->cells.nb(),
-                [this](Box& B, index_t c) {
-                    // Get cell bbox
-                    const double* p = mesh_->vertices.point_ptr(
-                        mesh_->cells.vertex(c,0)
-                    );
-                    for(coord_index_t coord = 0; coord < 3; ++coord) {
-                        B.xyz_min[coord] = p[coord];
-                        B.xyz_max[coord] = p[coord];
-                    }
-                    for(index_t lv=1; lv<mesh_->cells.nb_vertices(c); ++lv) {
-                        p = mesh_->vertices.point_ptr(
-                            mesh_->cells.vertex(c,lv)
-                        );
-                        for(coord_index_t coord = 0; coord < 3; ++coord) {
-                            B.xyz_min[coord] = std::min(
-                                B.xyz_min[coord], p[coord]
-                            );
-                            B.xyz_max[coord] = std::max(
-                                B.xyz_max[coord], p[coord]
-                            );
-                        }
-                    }
-                }
-            );
-        }
+	    break;
+	case AABB_INDIRECT:
+	    compute_mesh_elements_spatial_order(
+		*mesh_, MESH_CELLS, reorder_, MESH_ORDER_MORTON
+	    );
+	    break;
+	}
+	AABB::initialize(
+	    mesh_->cells.nb(),
+	    [this](Box& B, index_t c) {
+		// Get cell bbox
+		for(coord_index_t coord = 0; coord < 3; ++coord) {
+		    B.xyz_min[coord] = Numeric::max_float64();
+		    B.xyz_max[coord] = -Numeric::max_float64();
+		}
+		for(const vec3& p: mesh_->cells.points(c)) {
+		    for(coord_index_t coord = 0; coord < 3; ++coord) {
+			B.xyz_min[coord] = std::min(
+			    B.xyz_min[coord], p[coord]
+			);
+			B.xyz_max[coord] = std::max(
+			    B.xyz_max[coord], p[coord]
+			);
+		    }
+		}
+	    }
+	);
     }
 
     index_t MeshCellsAABB::containing_tet_recursive(
@@ -827,8 +815,9 @@ namespace GEO {
         }
 
         if(e==b+1) {
-            if(mesh_tet_contains_point(*mesh_, b, p)) {
-                return b;
+	    index_t t = element_in_leaf(b);
+            if(mesh_tet_contains_point(*mesh_, t, p)) {
+                return t;
             } else {
                 return NO_TET;
             }
@@ -868,61 +857,26 @@ namespace GEO {
         if(reorder) {
             mesh_reorder(*mesh_, MESH_ORDER_MORTON);
         }
-        if(mesh_->facets.are_simplices()) {
-            AABB::initialize(
-                mesh_->facets.nb(),
-                [this](Box2d& B, index_t t) {
-                    // Get triangle bbox
-                    const double* p = mesh_->vertices.point_ptr(
-                        mesh_->facets.vertex(t,0)
-                    );
-                    for(coord_index_t coord = 0; coord < 2; ++coord) {
-                        B.xy_min[coord] = p[coord];
-                        B.xy_max[coord] = p[coord];
-                    }
-                    for(index_t lv=1; lv<3; ++lv) {
-                        p = mesh_->vertices.point_ptr(
-                            mesh_->facets.vertex(t,lv)
-                        );
-                        for(coord_index_t coord = 0; coord < 2; ++coord) {
-                            B.xy_min[coord] = std::min(
-                                B.xy_min[coord], p[coord]
-                            );
-                            B.xy_max[coord] = std::max(
-                                B.xy_max[coord], p[coord]
-                            );
-                        }
-                    }
-                }
-            );
-        } else {
-            AABB::initialize(
-                mesh_->facets.nb(),
-                [this](Box2d& B, index_t f) {
-                    // Get facet bbox
-                    const double* p = mesh_->vertices.point_ptr(
-                        mesh_->facets.vertex(f,0)
-                    );
-                    for(coord_index_t coord = 0; coord < 2; ++coord) {
-                        B.xy_min[coord] = p[coord];
-                        B.xy_max[coord] = p[coord];
-                    }
-                    for(index_t lv=1; lv<mesh_->facets.nb_vertices(f); ++lv) {
-                        p = mesh_->vertices.point_ptr(
-                            mesh_->cells.vertex(f,lv)
-                        );
-                        for(coord_index_t coord = 0; coord < 2; ++coord) {
-                            B.xy_min[coord] = std::min(
-                                B.xy_min[coord], p[coord]
-                            );
-                            B.xy_max[coord] = std::max(
-                                B.xy_max[coord], p[coord]
-                            );
-                        }
-                    }
-                }
-            );
-        }
+	AABB::initialize(
+	    mesh_->facets.nb(),
+	    [this](Box2d& B, index_t f) {
+		// Get facet bbox
+		for(coord_index_t coord = 0; coord < 2; ++coord) {
+		    B.xy_min[coord] = Numeric::max_float64();
+		    B.xy_max[coord] = -Numeric::max_float64();
+		}
+		for(const vec2& p: mesh_->facets.points<2>(f)) {
+		    for(coord_index_t coord = 0; coord < 2; ++coord) {
+			B.xy_min[coord] = std::min(
+			    B.xy_min[coord], p[coord]
+			);
+			B.xy_max[coord] = std::max(
+			    B.xy_max[coord], p[coord]
+			);
+		    }
+		}
+	    }
+	);
         if(was_2d) {
             M.vertices.set_dimension(2);
         }
@@ -938,7 +892,8 @@ namespace GEO {
         }
 
         if(e==b+1) {
-            if(mesh_triangle_contains_point(*mesh_, b, p)) {
+	    index_t f = element_in_leaf(b);
+            if(mesh_triangle_contains_point(*mesh_, f, p)) {
                 return b;
             } else {
                 return NO_TRIANGLE;
