@@ -226,13 +226,13 @@ namespace GEO {
             index_t f1 = b->facets.adjacent(f,0);
             index_t f2 = b->facets.adjacent(f,1);
             index_t f3 = b->facets.adjacent(f,2);
-            if(f1 != index_t(-1)) {
+            if(f1 != NO_INDEX) {
                 f1 += f_ofs;
             }
-            if(f2 != index_t(-1)) {
+            if(f2 != NO_INDEX) {
                 f2 += f_ofs;
             }
-            if(f3 != index_t(-1)) {
+            if(f3 != NO_INDEX) {
                 f3 += f_ofs;
             }
             a->facets.set_adjacent(f + f_ofs, 0, f1);
@@ -253,7 +253,7 @@ namespace GEO {
                 bbox_.xyz_max[c], other->bbox().xyz_max[c]
             );
         }
-        if(operand != index_t(-1)) {
+        if(operand != NO_INDEX) {
             Attribute<index_t> f_operand_bit(facets.attributes(),"operand_bit");
             for(index_t f=f_ofs; f<facets.nb(); ++f) {
                 f_operand_bit[f] = index_t(1) << operand;
@@ -272,12 +272,15 @@ namespace GEO {
         reset_file_path();
         STL_epsilon_ = 1e-6;
         verbose_ = false;
+	fine_verbose_ = false;
         max_arity_ = 32;
         simplify_coplanar_facets_ = true;
         coplanar_angle_tolerance_ = 0.0;
         delaunay_ = true;
         detect_intersecting_neighbors_ = true;
         fast_union_ = false;
+	warnings_ = false;
+	noop_ = false;
     }
 
     void CSGBuilder::reset_defaults() {
@@ -303,16 +306,18 @@ namespace GEO {
         M->vertices.set_dimension(2);
 
         if(size.x <= 0.0 || size.y <= 0.0) {
-            Logger::warn("CSG")
-                << "square with negative size (returning empty shape)"
-                << std::endl;
+	    if(warnings_) {
+		Logger::warn("CSG")
+		    << "square with negative size (returning empty shape)"
+		    << std::endl;
+	    }
             return M;
         }
 
-        M->vertices.create_vertex(vec2(x1,y1).data());
-        M->vertices.create_vertex(vec2(x2,y1).data());
-        M->vertices.create_vertex(vec2(x1,y2).data());
-        M->vertices.create_vertex(vec2(x2,y2).data());
+        M->vertices.create_vertex(vec2(x1,y1));
+        M->vertices.create_vertex(vec2(x2,y1));
+        M->vertices.create_vertex(vec2(x1,y2));
+        M->vertices.create_vertex(vec2(x2,y2));
 
         M->facets.create_triangle(0,3,1);
         M->facets.create_triangle(0,2,3);
@@ -331,9 +336,11 @@ namespace GEO {
         M->vertices.set_dimension(2);
 
         if(r <= 0.0) {
-            Logger::warn("CSG")
-                << "circle with negative radius (returning empty shape)"
-                << std::endl;
+	    if(warnings_) {
+		Logger::warn("CSG")
+		    << "circle with negative radius (returning empty shape)"
+		    << std::endl;
+	    }
             return M;
         }
 
@@ -343,7 +350,7 @@ namespace GEO {
             double stheta = sin(theta);
             double x = ctheta*r;
             double y = stheta*r;
-            M->vertices.create_vertex(vec2(x,y).data());
+            M->vertices.create_vertex(vec2(x,y));
         }
 
         for(index_t u=1; u+1<nu; ++u) {
@@ -377,20 +384,22 @@ namespace GEO {
         M->vertices.set_dimension(3);
 
         if(size.x <= 0.0 || size.y <= 0.0 || size.z <= 0.0) {
-            Logger::warn("CSG")
-                << "cube with negative size (returning empty shape)"
-                << std::endl;
+	    if(warnings_) {
+		Logger::warn("CSG")
+		    << "cube with negative size (returning empty shape)"
+		    << std::endl;
+	    }
             return M;
         }
 
-        M->vertices.create_vertex(vec3(x1,y1,z1).data());
-        M->vertices.create_vertex(vec3(x2,y1,z1).data());
-        M->vertices.create_vertex(vec3(x1,y2,z1).data());
-        M->vertices.create_vertex(vec3(x2,y2,z1).data());
-        M->vertices.create_vertex(vec3(x1,y1,z2).data());
-        M->vertices.create_vertex(vec3(x2,y1,z2).data());
-        M->vertices.create_vertex(vec3(x1,y2,z2).data());
-        M->vertices.create_vertex(vec3(x2,y2,z2).data());
+        M->vertices.create_vertex(vec3(x1,y1,z1));
+        M->vertices.create_vertex(vec3(x2,y1,z1));
+        M->vertices.create_vertex(vec3(x1,y2,z1));
+        M->vertices.create_vertex(vec3(x2,y2,z1));
+        M->vertices.create_vertex(vec3(x1,y1,z2));
+        M->vertices.create_vertex(vec3(x2,y1,z2));
+        M->vertices.create_vertex(vec3(x1,y2,z2));
+        M->vertices.create_vertex(vec3(x2,y2,z2));
 
         M->facets.create_triangle(7,3,6);
         M->facets.create_triangle(6,3,2);
@@ -412,25 +421,28 @@ namespace GEO {
 
     CSGMesh_var CSGBuilder::sphere(double r) {
         index_t nu = get_fragments_from_r(r);
-        index_t nv = index_t((nu / 2) + 1); // TODO: I do not have exactly the
-                                            // same parameterization as OpenSCAD
-                                            // in v ...
+        index_t nv = nu / 2;
+	if(nu >= 5 && (nu & 1) != 0) {
+	    ++nv;
+	}
+
+	nu = std::max(nu, index_t(3));
+	nv = std::max(nv, index_t(2));
 
         CSGMesh_var M = new CSGMesh;
         M->vertices.set_dimension(3);
 
         if(r <= 0.0) {
-            Logger::warn("CSG")
-                << "sphere with negative radius (returning empty shape)"
-                << std::endl;
+	    if(warnings_) {
+		Logger::warn("CSG")
+		    << "sphere with negative radius (returning empty shape)"
+		    << std::endl;
+	    }
             return M;
         }
 
-        // First pole
-        M->vertices.create_vertex(vec3(0.0, 0.0, -r).data());
-        // All vertices except poles
-        for(index_t v=1; v<nv-1; ++v) {
-            double phi = double(v)*M_PI/double(nv-1) - M_PI/2.0;
+        for(index_t v=0; v<nv; ++v) {
+            double phi = (double(v) + 0.5)*M_PI/double(nv) - M_PI/2.0;
 
             double cphi = cos(phi);
             double sphi = sin(phi);
@@ -441,22 +453,13 @@ namespace GEO {
                 double x = r*ctheta*cphi;
                 double y = r*stheta*cphi;
                 double z = r*sphi;
-                M->vertices.create_vertex(vec3(x,y,z).data());
+                M->vertices.create_vertex(vec3(x,y,z));
             }
         }
-        // Second pole
-        M->vertices.create_vertex(vec3(0.0, 0.0, r).data());
 
-        // Maps param coordinates to mesh index, taking into
-        // account poles (at v=0 and v=nv-1)
-        auto vindex = [nu,nv](index_t u, index_t v)->index_t {
-            if(v==0) {
-                return 0;
-            }
-            if(v==(nv-1)) {
-                return 1 + (nv-2)*nu;
-            }
-            return 1+(v-1)*nu+(u%nu); // make u wraparound
+        // Maps param coordinates to mesh index
+        auto vindex = [nu](index_t u, index_t v)->index_t {
+            return v*nu+(u%nu); // make u wraparound
         };
 
         for(index_t v=0; v<nv-1; ++v) {
@@ -465,17 +468,36 @@ namespace GEO {
                 index_t v10 = vindex(u+1,v  );
                 index_t v01 = vindex(u  ,v+1);
                 index_t v11 = vindex(u+1,v+1);
-                // Create triangles, skip degenerate triangles
-                // around the poles.
-                if(v01 != v11) {
-                    M->facets.create_triangle(v00, v11, v01);
-                }
-                if(v00 != v10) {
-                    M->facets.create_triangle(v00, v10, v11);
-                }
+		M->facets.create_triangle(v00, v10, v01);
+		M->facets.create_triangle(v10, v11, v01);
             }
         }
 
+	// Cappings with zigzag triangulation
+	{
+	    index_t offset = nu*(nv-1);
+	    index_t low=0;
+	    index_t high=nu-1;
+	    for(;;) {
+		if(low+1==high) {
+		    break;
+		}
+		M->facets.create_triangle(low+1, low, high);
+		M->facets.create_triangle(
+		    offset+low, offset+low+1, offset+high
+		);
+		++low;
+		if(high-1==low) {
+		    break;
+		}
+		M->facets.create_triangle(low, high, high-1);
+		M->facets.create_triangle(
+		    offset+low, offset+high-1, offset+high
+		);
+		--high;
+	    }
+	}
+
         M->facets.connect();
         M->update_bbox();
         return M;
@@ -493,9 +515,11 @@ namespace GEO {
         M->vertices.set_dimension(3);
 
         if(r1 < 0.0 || r2 < 0.0) {
-            Logger::warn("CSG")
-                << "cylinder with negative radius (returning empty shape)"
-                << std::endl;
+	    if(warnings_) {
+		Logger::warn("CSG")
+		    << "cylinder with negative radius (returning empty shape)"
+		    << std::endl;
+	    }
             return M;
         }
 
@@ -510,11 +534,11 @@ namespace GEO {
             double stheta = sin(theta);
             double x = ctheta*r1;
             double y = stheta*r1;
-            M->vertices.create_vertex(vec3(x,y,z1).data());
+            M->vertices.create_vertex(vec3(x,y,z1));
         }
 
         if(r2 == 0.0) {
-            M->vertices.create_vertex(vec3(0.0, 0.0, z2).data());
+            M->vertices.create_vertex(vec3(0.0, 0.0, z2));
         } else {
             for(index_t u=0; u<nu; ++u) {
                 double theta = double(u)*2.0*M_PI/double(nu);
@@ -522,7 +546,7 @@ namespace GEO {
                 double stheta = sin(theta);
                 double x = ctheta*r2;
                 double y = stheta*r2;
-                M->vertices.create_vertex(vec3(x,y,z2).data());
+                M->vertices.create_vertex(vec3(x,y,z2));
             }
         }
 
@@ -672,7 +696,7 @@ namespace GEO {
         for(index_t f: result->facets) {
             for(index_t lv=0; lv<result->facets.nb_vertices(f); ++lv) {
                 index_t v = result->facets.vertex(f,lv);
-                if(result->vertices.point_ptr(v)[2] != 0.0) {
+                if(result->vertices.point(v).z != 0.0) {
                     delete_f[f] = 1;
                 }
             }
@@ -727,7 +751,7 @@ namespace GEO {
         result->vertices.create_vertices(image->width() * image->height());
         for(index_t v=0; v < nv; ++v) {
             for(index_t u=0; u<nu; ++u) {
-                double* p = result->vertices.point_ptr(v*nu+u);
+                vec3& p = result->vertices.point(v*nu+u);
                 double x = double(u);
                 double y = double(v);
                 double z = image->pixel_base_float64_ptr(u,v)[0];
@@ -756,12 +780,12 @@ namespace GEO {
                 index_t v10 = v*nu+u+1;
                 index_t v01 = (v+1)*nu+u;
                 index_t v11 = (v+1)*nu+u+1;
-                vec3 p00(result->vertices.point_ptr(v00));
-                vec3 p10(result->vertices.point_ptr(v10));
-                vec3 p01(result->vertices.point_ptr(v01));
-                vec3 p11(result->vertices.point_ptr(v11));
+                vec3 p00 = result->vertices.point(v00);
+                vec3 p10 = result->vertices.point(v10);
+                vec3 p01 = result->vertices.point(v01);
+                vec3 p11 = result->vertices.point(v11);
                 vec3 p = 0.25*(p00+p10+p01+p11);
-                index_t w = result->vertices.create_vertex(p.data());
+                index_t w = result->vertices.create_vertex(p);
                 result->facets.create_triangle(v00,v10,w);
                 result->facets.create_triangle(v10,v11,w);
                 result->facets.create_triangle(v11,v01,w);
@@ -771,17 +795,16 @@ namespace GEO {
 
         result->facets.connect();
 
-        vector<index_t> projected(result->vertices.nb(), index_t(-1));
+        vector<index_t> projected(result->vertices.nb(), NO_INDEX);
         for(index_t f: result->facets) {
             for(index_t le=0; le<3; ++le) {
-                if(result->facets.adjacent(f,le) == index_t(-1)) {
+                if(result->facets.adjacent(f,le) == NO_INDEX) {
                     for(index_t dle=0; dle<2; ++dle) {
                         index_t v = result->facets.vertex(f, (le+dle)%3);
-                        if(projected[v] == index_t(-1)) {
-                            vec3 p(result->vertices.point_ptr(v));
+                        if(projected[v] == NO_INDEX) {
+                            vec3 p = result->vertices.point(v);
                             p.z = z1;
-                            projected[v] =
-				result->vertices.create_vertex(p.data());
+                            projected[v] = result->vertices.create_vertex(p);
                         }
                     }
                 }
@@ -790,7 +813,7 @@ namespace GEO {
 
         for(index_t f: result->facets) {
             for(index_t le=0; le<3; ++le) {
-                if(result->facets.adjacent(f,le) == index_t(-1)) {
+                if(result->facets.adjacent(f,le) == NO_INDEX) {
                     index_t v1 = result->facets.vertex(f,le);
                     index_t v2 = result->facets.vertex(f,(le+1)%3);
                     result->facets.create_triangle(
@@ -810,11 +833,79 @@ namespace GEO {
         return result;
     }
 
+    CSGMesh_var CSGBuilder::text_with_OpenSCAD(
+	const std::string& text,
+	double size,
+	const std::string& font,
+	const std::string& halign,
+	const std::string& valign,
+	double spacing,
+	const std::string& direction,
+	const std::string& language,
+	const std::string& script
+    ) {
+        Logger::out("CSG") << "Handling text() with OpenSCAD" << std::endl;
+
+	if(text == "") {
+	    CSGMesh_var result = new CSGMesh;
+	    return result;
+	}
+
+        // Generate a simple linear extrusion, so that we can convert to STL
+        // (without it OpenSCAD refuses to create a STL with 2D content)
+        std::ofstream tmp("tmpscad.scad");
+        tmp << "group() {" << std::endl;
+        tmp << "   linear_extrude(height=1.0) {" << std::endl;
+        tmp << "      text(" << std::endl;
+	tmp << "             \"" << text << "\"," << std::endl;
+        tmp << "             size=" << size << "," << std::endl;
+	if(font != "") {
+	    tmp << "             font=\"" << font << "\"," << std::endl;
+	}
+	tmp << "             halign=\"" << halign << "\"," << std::endl;
+	tmp << "             valign=\"" << valign << "\"," << std::endl;
+	tmp << "             spacing=" << spacing << "," << std::endl;
+	tmp << "             direction=\"" << direction << "\"," << std::endl;
+	tmp << "             language=\"" << language<< "\"," << std::endl;
+	tmp << "             script=\"" << script << "\"" << std::endl;
+        tmp << "      );" << std::endl;
+        tmp << "   }" << std::endl;
+        tmp << "}" << std::endl;
+
+        // Start OpenSCAD and generate output as STL
+        if(system("openscad tmpscad.scad -o tmpscad.stl")) {
+            Logger::warn("CSG") << "Error while running openscad " << std::endl;
+            Logger::warn("CSG") << "(used to generate text) " << std::endl;
+        }
+
+        // Load STL using our own loader
+        CSGMesh_var result = import("tmpscad.stl");
+
+
+	FileSystem::delete_file("tmpscad.scad");
+	FileSystem::delete_file("tmpscad.stl");
+
+        // Delete the facets that are coming from the linear extrusion
+        vector<index_t> delete_f(result->facets.nb(),0);
+        for(index_t f: result->facets) {
+            for(index_t lv=0; lv<result->facets.nb_vertices(f); ++lv) {
+                index_t v = result->facets.vertex(f,lv);
+                if(result->vertices.point(v).z != 0.0) {
+                    delete_f[f] = 1;
+                }
+            }
+        }
+        result->facets.delete_elements(delete_f);
+        result->vertices.set_dimension(2);
+        return result;
+    }
+
+
     Image* CSGBuilder::load_dat_image(const std::string& file_name) {
         LineInput in(file_name);
 
-        index_t nrows  = index_t(-1);
-        index_t ncols  = index_t(-1);
+        index_t nrows  = NO_INDEX;
+        index_t ncols  = NO_INDEX;
         Image* result = nullptr;
 
         try {
@@ -899,6 +990,9 @@ namespace GEO {
     }
 
     CSGMesh_var CSGBuilder::union_instr(const CSGScope& scope) {
+	if(noop_) {
+	    return append(scope);
+	}
         if(scope.size() == 1) {
             return scope[0];
         }
@@ -945,6 +1039,9 @@ namespace GEO {
 
 
     CSGMesh_var CSGBuilder::intersection(const CSGScope& scope) {
+	if(noop_) {
+	    return append(scope);
+	}
         if(scope.size() == 1) {
             return scope[0];
         }
@@ -978,6 +1075,10 @@ namespace GEO {
     }
 
     CSGMesh_var CSGBuilder::difference(const CSGScope& scope) {
+	if(noop_) {
+	    return append(scope);
+	}
+
         if(scope.size() == 1) {
             return scope[0];
         }
@@ -1015,7 +1116,11 @@ namespace GEO {
         }
 
         CSGMesh_var result = new CSGMesh;
-        result->vertices.set_dimension(3);
+	index_t dim = 2;
+        for(index_t i=0; i<scope.size(); ++i) {
+	    dim = std::max(dim, scope[i]->vertices.dimension());
+	}
+        result->vertices.set_dimension(dim);
 
         if(!fast_union_ && scope.size() > max_arity_) {
             Logger::warn("CSG") << "Scope with more than "
@@ -1025,7 +1130,9 @@ namespace GEO {
         }
 
         for(index_t i=0; i<scope.size(); ++i) {
-            result->append_mesh(scope[i], i);
+	    if(scope[i]->vertices.nb() > 0) {
+		result->append_mesh(scope[i], i);
+	    }
         }
 
         return result;
@@ -1057,6 +1164,15 @@ namespace GEO {
             }
         }
 
+        CSGMesh_var result = new CSGMesh;
+        result->vertices.set_dimension(dim);
+
+	// Particular case: no vertex in scope (yes, this happens !)
+	if(points.size() == 0) {
+	    return result;
+	}
+
+
         if(dim == 3) {
             CmdLine::set_arg("algo:delaunay", "PDEL");
         } else {
@@ -1066,25 +1182,39 @@ namespace GEO {
         delaunay->set_keeps_infinite(true);
         delaunay->set_vertices(nb_pts, points.data());
 
-        CSGMesh_var result = new CSGMesh;
-        result->vertices.set_dimension(dim);
 
         if(dim == 3) {
             vector<index_t> tri2v;
             // This iterates on the infinite cells
             for(
                 index_t t = delaunay->nb_finite_cells();
-                t < delaunay->nb_cells(); ++t
+		t < delaunay->nb_cells(); ++t
             ) {
-                for(index_t lv=0; lv<4; ++lv) {
-                    signed_index_t v = delaunay->cell_vertex(t,lv);
-                    if(v != -1) {
-                        tri2v.push_back(index_t(v));
-                    }
-                }
-            }
+		index_t v0 = delaunay->cell_vertex(t,0);
+		index_t v1 = delaunay->cell_vertex(t,1);
+		index_t v2 = delaunay->cell_vertex(t,2);
+		index_t v3 = delaunay->cell_vertex(t,3);
+		if(v0 == NO_INDEX) {
+		    tri2v.push_back(v1);
+		    tri2v.push_back(v2);
+		    tri2v.push_back(v3);
+		} else if(v1 == NO_INDEX) {
+		    tri2v.push_back(v3);
+		    tri2v.push_back(v2);
+		    tri2v.push_back(v0);
+		} else if(v2 == NO_INDEX) {
+		    tri2v.push_back(v1);
+		    tri2v.push_back(v3);
+		    tri2v.push_back(v0);
+		} else if(v3 == NO_INDEX) {
+		    tri2v.push_back(v2);
+		    tri2v.push_back(v1);
+		    tri2v.push_back(v0);
+		}
+	    }
             result->facets.assign_triangle_mesh(3, points, tri2v, true);
             result->vertices.remove_isolated();
+	    result->facets.connect();
         } else {
             result->vertices.set_dimension(2);
             result->vertices.create_vertices(nb_pts);
@@ -1095,15 +1225,15 @@ namespace GEO {
             for(index_t t = delaunay->nb_finite_cells();
                 t < delaunay->nb_cells(); ++t
                ) {
-                signed_index_t v1=-1,v2=-1;
+                index_t v1= NO_INDEX, v2=NO_INDEX;
                 for(index_t lv=0; lv<3; ++lv) {
-                    if(delaunay->cell_vertex(t,lv) == -1) {
+                    if(delaunay->cell_vertex(t,lv) == NO_INDEX) {
                         v1 = delaunay->cell_vertex(t,(lv+1)%3);
                         v2 = delaunay->cell_vertex(t,(lv+2)%3);
                     }
                 }
-                geo_assert(v1 != -1 && v2 != -1);
-                result->edges.create_edge(index_t(v1),index_t(v2));
+                geo_assert(v1 != NO_INDEX && v2 != NO_INDEX);
+                result->edges.create_edge(v1,v2);
             }
             result->vertices.remove_isolated();
         }
@@ -1119,10 +1249,9 @@ namespace GEO {
         double z2 = center ?  height/2.0 : height;
 
         CSGMesh_var M = scope.size() == 1 ? scope[0] : group(scope);
+
         if(M->vertices.dimension() != 2) {
-            throw(std::logic_error(
-                      "linear_extrude: mesh is not of dimension 2"
-                  ));
+	    M->vertices.set_dimension(2);
         }
         if(M->facets.nb() == 0) {
             triangulate(M,"union");
@@ -1137,14 +1266,14 @@ namespace GEO {
         // Reorder vertices so that border vertices come first, then internal
         // vertices
         {
-            vector<index_t> reorder_vertices(M->vertices.nb(), index_t(-1));
+            vector<index_t> reorder_vertices(M->vertices.nb(), NO_INDEX);
             for(index_t f: M->facets) {
                 for(index_t le=0; le<3; ++le) {
-                    if(M->facets.adjacent(f,le) == index_t(-1)) {
+                    if(M->facets.adjacent(f,le) == NO_INDEX) {
                         index_t v = M->facets.vertex(f,le);
                         // There may be non-manifold borders incident to
                         // the same border vertex several times.
-                        if(reorder_vertices[v] != index_t(-1)) {
+                        if(reorder_vertices[v] != NO_INDEX) {
                             continue;
                         }
                         reorder_vertices[v] = nv_border;
@@ -1153,7 +1282,7 @@ namespace GEO {
                 }
             }
             for(index_t v: M->vertices) {
-                if(reorder_vertices[v] == index_t(-1)) {
+                if(reorder_vertices[v] == NO_INDEX) {
                     reorder_vertices[v] = nv_intern + nv_border;
                     ++nv_intern;
                 }
@@ -1163,8 +1292,8 @@ namespace GEO {
         geo_assert(nv_border + nv_intern == nv);
 
         // Set z coordinates of all original vertices to z1
-        for(index_t v: M->vertices) {
-            M->vertices.point_ptr(v)[2] = z1;
+        for(vec3& p: M->vertices.points()) {
+            p.z = z1;
         }
 
         if(slices == 0) {
@@ -1173,7 +1302,7 @@ namespace GEO {
         if(slices == 0 && twist != 0.0) {
             double R = 0;
             for(index_t v: M->vertices) {
-                const double* p = M->vertices.point_ptr(v);
+                vec2 p = M->vertices.point<2>(v);
                 R = std::max(R, geo_sqr(p[0])+geo_sqr(p[1]));
                 R = std::max(R, geo_sqr(p[0]*scale.x)+geo_sqr(p[1]*scale.y));
             }
@@ -1186,14 +1315,14 @@ namespace GEO {
         index_t border_offset = first_border_offset;
 
         auto extrude_vertex = [&](index_t to_v, index_t from_v, double t) {
-            const double* ref = M->vertices.point_ptr(from_v);
-            double* target = M->vertices.point_ptr(to_v);
+            const vec2& ref = M->vertices.point<2>(from_v);
+
             double s = 1.0 - t;
             double z = s*z1 + t*z2;
             vec2   sz = s*vec2(1.0, 1.0) + t*scale;
 
-            double x = ref[0] * sz.x;
-            double y = ref[1] * sz.y;
+            double x = ref.x * sz.x;
+            double y = ref.y * sz.y;
 
             if(twist != 0.0) {
                 double alpha = twist*t*M_PI/180.0;
@@ -1205,9 +1334,7 @@ namespace GEO {
                 y = y2;
             }
 
-            target[0] = x;
-            target[1] = y;
-            target[2] = z;
+	    M->vertices.point(to_v) = {x,y,z};
         };
 
         for(index_t Z=1; Z<=slices; ++Z) {
@@ -1215,11 +1342,10 @@ namespace GEO {
 
             // Special case: scaling = 0 (create a pole)
             if(Z == slices && scale.x == 0.0 && scale.y == 0.0) {
-                double p[3] = {0.0, 0.0, z2};
-                index_t pole = M->vertices.create_vertex(p);
+                index_t pole = M->vertices.create_vertex(vec3{0.0, 0.0, z2});
                 for(index_t f=0; f<nf; ++f) {
                     for(index_t le=0; le<3; ++le) {
-                        if(M->facets.adjacent(f,le) == index_t(-1)) {
+                        if(M->facets.adjacent(f,le) == NO_INDEX) {
                             index_t v1 = M->facets.vertex(f,le);
                             index_t v2 = M->facets.vertex(f,(le+1)%3);
                             v1 += border_offset;
@@ -1246,7 +1372,7 @@ namespace GEO {
             // Create walls
             for(index_t f=0; f<nf; ++f) {
                 for(index_t le=0; le<3; ++le) {
-                    if(M->facets.adjacent(f,le) == index_t(-1)) {
+                    if(M->facets.adjacent(f,le) == NO_INDEX) {
                         index_t v1 = M->facets.vertex(f,le);
                         index_t v2 = M->facets.vertex(f,(le+1)%3);
                         index_t w1 = v1 + next_border_offset;
@@ -1331,14 +1457,14 @@ namespace GEO {
         // Reorder vertices so that border vertices come first, then internal
         // vertices
         if(M->facets.nb() != 0) {
-            vector<index_t> reorder_vertices(M->vertices.nb(), index_t(-1));
+            vector<index_t> reorder_vertices(M->vertices.nb(), NO_INDEX);
             for(index_t f: M->facets) {
                 for(index_t le=0; le<3; ++le) {
-                    if(M->facets.adjacent(f,le) == index_t(-1)) {
+                    if(M->facets.adjacent(f,le) == NO_INDEX) {
                         index_t v = M->facets.vertex(f,le);
                         // There may be non-manifold borders incident to
                         // the same border vertex several times.
-                        if(reorder_vertices[v] != index_t(-1)) {
+                        if(reorder_vertices[v] != NO_INDEX) {
                             continue;
                         }
                         reorder_vertices[v] = nv_border;
@@ -1347,7 +1473,7 @@ namespace GEO {
                 }
             }
             for(index_t v: M->vertices) {
-                if(reorder_vertices[v] == index_t(-1)) {
+                if(reorder_vertices[v] == NO_INDEX) {
                     reorder_vertices[v] = nv_intern + nv_border;
                     ++nv_intern;
                 }
@@ -1360,15 +1486,12 @@ namespace GEO {
         geo_assert(nv_border + nv_intern == nv);
 
         auto extrude_vertex = [&](index_t to_v, index_t from_v, double t) {
-            const double* ref = M->vertices.point_ptr(from_v);
-            double* target = M->vertices.point_ptr(to_v);
+            vec2 ref = M->vertices.point<2>(from_v);
             double alpha = t * 2.0 * M_PI * angle / 360.0;
-            double x = ref[0] * cos(alpha);
-            double y = ref[0] * sin(alpha);
-            double z = ref[1];
-            target[0] = x;
-            target[1] = y;
-            target[2] = z;
+            double x = ref.x * cos(alpha);
+            double y = ref.x * sin(alpha);
+            double z = ref.y;
+	    M->vertices.point(to_v) = {x, y, z};
         };
 
 
@@ -1512,17 +1635,9 @@ namespace GEO {
             {
                 CSGScope scope2;
                 for(index_t f: result->facets) {
-                    const double* p1 = result->vertices.point_ptr(
-                        result->facets.vertex(f,0)
-                    );
-
-                    const double* p2 = result->vertices.point_ptr(
-                        result->facets.vertex(f,1)
-                    );
-
-                    const double* p3 = result->vertices.point_ptr(
-                        result->facets.vertex(f,2)
-                    );
+		    const vec2& p1 = result->facets.point<2>(f,0);
+		    const vec2& p2 = result->facets.point<2>(f,1);
+		    const vec2& p3 = result->facets.point<2>(f,2);
 
                     // I thought that I could have said here: != POSITIVE
                     // NOTE: different set of isolated vertices each time,
@@ -1556,6 +1671,7 @@ namespace GEO {
         } else {
             MeshSurfaceIntersection I(*mesh);
             I.set_verbose(verbose_);
+	    I.set_fine_verbose(fine_verbose_);
             I.set_delaunay(delaunay_);
             I.set_detect_intersecting_neighbors(detect_intersecting_neighbors_);
             if(fast_union_ && boolean_expr == "union") {
@@ -1609,7 +1725,7 @@ namespace GEO {
         // In case there are duplicated vertices, keep track of indexing
         vector<index_t> vertex_id(mesh->vertices.nb());
         for(index_t v: mesh->vertices) {
-            vec2 p(mesh->vertices.point_ptr(v));
+            vec2 p = mesh->vertices.point<2>(v);
             vertex_id[v] = CDT.insert(ExactCDT2d::ExactPoint(p),v);
         }
 
@@ -1628,14 +1744,14 @@ namespace GEO {
             }
         }
 
-        CDT.classify_triangles(boolean_expr);
+	CDT.classify_triangles(boolean_expr);
 
         // Create vertices coming from constraint intersections
         for(index_t v=nv0; v<CDT.nv(); ++v) {
             vec2 p = PCK::approximate(CDT.point(v));
             CDT.set_vertex_id(
                 v,
-                mesh->vertices.create_vertex(p.data())
+                mesh->vertices.create_vertex(p)
             );
         }
 
@@ -1701,6 +1817,7 @@ namespace GEO {
         DECLARE_OBJECT(polygon);
         DECLARE_OBJECT(import);
         DECLARE_OBJECT(surface);
+        DECLARE_OBJECT(text);
 
 #define DECLARE_INSTRUCTION(instr)                              \
         instruction_funcs_[#instr] = &CSGCompiler::instr;
@@ -1719,6 +1836,12 @@ namespace GEO {
     }
 
     CSGMesh_var CSGCompiler::compile_file(const std::string& input_filename) {
+
+        // Add the directory that contains the file to the builder's file path,
+        // so that import() instructions are able to find files in the same
+        // directory.
+        builder_.push_file_path(FileSystem::dir_name(input_filename));
+
         if(
             FileSystem::extension(input_filename) == "scad" ||
             FileSystem::extension(input_filename) == "SCAD"
@@ -1728,8 +1851,11 @@ namespace GEO {
             Logger::out("CSG") << "Converting scad file using openscad"
                                << std::endl;
 
+	    std::string tmpscad = FileSystem::get_current_working_directory() +
+		"/tmpscad.csg";
+
             // Ask openscad for help for parsing .scad files !
-            std::string command = "openscad "+input_filename+" -o tmpscad.csg";
+            std::string command = "openscad "+input_filename+" -o " + tmpscad;
 
             if(system(command.c_str())) {
                 Logger::err("CSG") << "Error while running openscad "
@@ -1739,9 +1865,10 @@ namespace GEO {
                 return result;
             }
 
-            result = compile_file("tmpscad.csg");
-            FileSystem::delete_file("tmpscad.csg");
-            return result;
+            result = compile_file(tmpscad);
+            FileSystem::delete_file(tmpscad);
+	    builder_.pop_file_path();
+	    return result;
         }
 
         filename_ = input_filename;
@@ -1763,17 +1890,13 @@ namespace GEO {
             );
         }
 
-        // Add the directory that contains the file to the builder's file path,
-        // so that import() instructions are able to find files in the same
-        // directory.
-        builder_.add_file_path(FileSystem::dir_name(input_filename));
         CSGMesh_var result = compile_string(source);
-        builder_.reset_file_path();
 
         if(!result.is_null() && result->vertices.dimension() == 2) {
             result->vertices.set_dimension(3);
         }
 
+        builder_.pop_file_path();
         return result;
     }
 
@@ -1890,9 +2013,11 @@ namespace GEO {
             if(points.array_val[v].size() != 3) {
                 syntax_error("polyhedron: wrong vertex size (expected 3d)");
             }
-            M->vertices.point_ptr(v)[0] = points.array_val[v][0];
-            M->vertices.point_ptr(v)[1] = points.array_val[v][1];
-            M->vertices.point_ptr(v)[2] = points.array_val[v][2];
+	    M->vertices.point(v) = {
+		points.array_val[v][0],
+		points.array_val[v][1],
+		points.array_val[v][2]
+	    };
         }
 
         for(index_t f=0; f<faces.array_val.size(); ++f) {
@@ -1923,24 +2048,32 @@ namespace GEO {
 
     CSGMesh_var CSGCompiler::polygon(const ArgList& args) {
         CSGMesh_var M = new CSGMesh;
+	M->vertices.set_dimension(2);
+
         if(!args.has_arg("points") || !args.has_arg("paths")) {
             syntax_error("polyhedron: missing points or paths");
         }
 
         const Value& points = args.get_arg("points");
 
+	if(points.type == Value::ARRAY1D && points.array_val.size() == 0) {
+	    // Special case, empty array, happens with BOSL2 scripts
+	    return M;
+	}
+
         if(points.type != Value::ARRAY2D) {
             syntax_error("polyhedron: wrong points type (expected array)");
         }
 
-        M->vertices.set_dimension(2);
         M->vertices.create_vertices(points.array_val.size());
         for(index_t v=0; v<points.array_val.size(); ++v) {
             if(points.array_val[v].size() != 2) {
                 syntax_error("polyhedron: wrong vertex size (expected 2d)");
             }
-            M->vertices.point_ptr(v)[0] = points.array_val[v][0];
-            M->vertices.point_ptr(v)[1] = points.array_val[v][1];
+            M->vertices.point<2>(v) = {
+		points.array_val[v][0],
+		points.array_val[v][1]
+	    };
         }
 
         const Value& paths = args.get_arg("paths");
@@ -2003,6 +2136,26 @@ namespace GEO {
         return builder_.surface(filename, center, invert);
     }
 
+
+    CSGMesh_var CSGCompiler::text(const ArgList& args) {
+	std::string text = args.get_arg("text", "");
+	if(text == "") {
+	    text = args.get_arg("arg_0", "");
+	}
+	double size = args.get_arg("size", 10.0);
+	std::string font = args.get_arg("font", "");
+	std::string halign = args.get_arg("halign", "left");
+	std::string valign = args.get_arg("valign", "baseline");
+	double spacing = args.get_arg("spacing", 1.0);
+	std::string direction = args.get_arg("direction", "ltr");
+	std::string language = args.get_arg("language", "en");
+	std::string script = args.get_arg("script", "latin");
+	return builder_.text_with_OpenSCAD(
+	    text, size, font, halign, valign,
+	    spacing, direction, language, script
+	);
+    }
+
     /********* Instructions **************************************************/
 
     CSGMesh_var CSGCompiler::multmatrix(
