@@ -779,8 +779,8 @@ namespace {
                 geo_debug_assert(e >= b);
                 geo_cite_with_info(
                     "WEB:SpatialSorting",
-                    "The implementation of spatial sort in GEOGRAM is inspired by "
-                    "the idea of using \\verb|std::nth_element()| and the recursive"
+                    "The implementation of spatial sort is inspired by "
+                    "the use of \\verb|std::nth_element()| and the recursive"
                     " template in the spatial sort package of CGAL"
                 );
 
@@ -797,8 +797,11 @@ namespace {
 
                 // Parallel sorting (2 then 4 then 8 sorts in parallel)
 
-// Unfortunately we cannot access consts for template arguments in lambdas in all
-// compilers (gcc is OK but not MSVC) so I'm using (ugly) macros here...
+                // Unfortunately we cannot access consts/constexprs for template
+                // arguments in lambdas in all compilers (gcc/clang OK but not
+                // MSVC) so I'm using macros here (it is ugly, but it is not a
+                // big drama), and I prefer that instead of hardwired constants
+		// that would make the code more difficult to read.
 
 #          define COORDX 0
 #          define COORDY 1
@@ -841,6 +844,7 @@ namespace {
 #          undef UPX
 #          undef UPY
 #          undef UPZ
+
             }
 
     private:
@@ -929,8 +933,8 @@ namespace {
                 geo_debug_assert(e > b);
                 geo_cite_with_info(
                     "WEB:SpatialSorting",
-                    "The implementation of spatial sort in GEOGRAM is inspired by "
-                    "the idea of using \\verb|std::nth_element()| and the recursive"
+                    "The implementation of spatial sort is inspired by "
+                    "the use of \\verb|std::nth_element()| and the recursive"
                     " template in the spatial sort package of CGAL"
                 );
 
@@ -1204,6 +1208,52 @@ namespace GEO {
         }
     }
 
+    void compute_mesh_elements_spatial_order(
+	const Mesh& M, MeshElementsFlags elements,
+	vector<index_t>& sorted_indices, MeshOrder order
+    ) {
+	geo_assert(M.vertices.dimension() >= 3);
+	switch(elements) {
+	case MESH_VERTICES: {
+            switch(order) {
+            case MESH_ORDER_HILBERT:
+                hilbert_vsort_3d(M, sorted_indices);
+                break;
+            case MESH_ORDER_MORTON:
+                morton_vsort_3d(M, sorted_indices);
+                break;
+            }
+	} break;
+	case MESH_FACETS: {
+	    switch(order) {
+	    case MESH_ORDER_HILBERT:
+		hilbert_fsort_3d(M, sorted_indices);
+		break;
+	    case MESH_ORDER_MORTON:
+		morton_fsort_3d(M, sorted_indices);
+		break;
+	    }
+	} break;
+	case MESH_CELLS: {
+            switch(order) {
+            case MESH_ORDER_HILBERT:
+                hilbert_csort_3d(M, sorted_indices);
+                break;
+            case MESH_ORDER_MORTON:
+                morton_csort_3d(M, sorted_indices);
+                break;
+            }
+	} break;
+	case MESH_NONE:
+        case MESH_EDGES:
+	case MESH_ALL_ELEMENTS:
+        case MESH_FACET_CORNERS:
+        case MESH_CELL_CORNERS:
+	case MESH_CELL_FACETS:
+        case MESH_ALL_SUBELEMENTS:
+	    geo_assert_not_reached;
+	}
+    }
 #endif
 
 
