@@ -45,9 +45,9 @@ namespace GEO {
 
     void GEOGRAM_API mesh_smooth(Mesh& M) {
         // Chain corners around vertices
-        vector<index_t> v2c(M.vertices.nb(), index_t(-1));
-        vector<index_t> next_c_around_v(M.facet_corners.nb(), index_t(-1));
-        vector<index_t> c2f(M.facet_corners.nb(), index_t(-1));
+        vector<index_t> v2c(M.vertices.nb(), NO_INDEX);
+        vector<index_t> next_c_around_v(M.facet_corners.nb(), NO_INDEX);
+        vector<index_t> c2f(M.facet_corners.nb(), NO_INDEX);
         for(index_t f: M.facets) {
             for(index_t c: M.facets.corners(f)) {
                 index_t v = M.facet_corners.vertex(c);
@@ -89,7 +89,7 @@ namespace GEO {
         for(index_t v: M.vertices) {
             nlBegin(NL_ROW);
             index_t count = 0;
-            for(index_t c = v2c[v]; c != index_t(-1); c = next_c_around_v[c]) {
+            for(index_t c = v2c[v]; c != NO_INDEX; c = next_c_around_v[c]) {
                 index_t f = c2f[c];
                 index_t c2 = M.facets.next_corner_around_facet(f,c);
                 index_t w = M.facet_corners.vertex(c2);
