@@ -43,6 +43,7 @@
 #include <geogram/mesh/mesh_repair.h>
 #include <geogram/mesh/mesh_fill_holes.h>
 #include <geogram/mesh/mesh_geometry.h>
+#include <geogram/mesh/mesh_topology.h>
 #include <geogram/mesh/mesh_io.h>
 #include <geogram/mesh/index.h>
 #include <geogram/delaunay/CDT_2d.h>
@@ -74,12 +75,9 @@ namespace {
     void remove_degenerate_triangles(Mesh& M) {
         vector<index_t> remove_f(M.facets.nb());
         for(index_t f: M.facets) {
-            index_t v1 = M.facets.vertex(f,0);
-            index_t v2 = M.facets.vertex(f,1);
-            index_t v3 = M.facets.vertex(f,2);
-            const double* p1 = M.vertices.point_ptr(v1);
-            const double* p2 = M.vertices.point_ptr(v2);
-            const double* p3 = M.vertices.point_ptr(v3);
+	    const vec3& p1 = M.facets.point(f,0);
+	    const vec3& p2 = M.facets.point(f,1);
+	    const vec3& p3 = M.facets.point(f,2);
             remove_f[f] = PCK::aligned_3d(p1,p2,p3);
         }
         M.facets.delete_elements(remove_f);
@@ -95,30 +93,7 @@ namespace {
         Mesh& M, const std::string& attribute = "chart"
     ) {
         Attribute<index_t> chart(M.facets.attributes(), attribute);
-        for(index_t f: M.facets) {
-            chart[f] = index_t(-1);
-        }
-        std::stack<index_t> S;
-        index_t cur_chart = 0;
-        for(index_t f: M.facets) {
-            if(chart[f] == index_t(-1)) {
-                chart[f] = cur_chart;
-                S.push(f);
-                while(!S.empty()) {
-                    index_t g = S.top();
-                    S.pop();
-                    for(index_t le=0; le<M.facets.nb_vertices(g); ++le) {
-                        index_t h = M.facets.adjacent(g,le);
-                        if(h != index_t(-1) && chart[h] == index_t(-1)) {
-                            chart[h] = cur_chart;
-                            S.push(h);
-                        }
-                    }
-                }
-                ++cur_chart;
-            }
-        }
-        return cur_chart;
+	return GEO::get_connected_components(M, chart);
     }
 
     /**
@@ -136,12 +111,12 @@ namespace {
     ) {
         geo_debug_assert(M.facets.nb_vertices(f1) == 3);
         geo_debug_assert(M.facets.nb_vertices(f2) == 3);
-        vec3 p1(M.vertices.point_ptr(M.facets.vertex(f1,0)));
-        vec3 p2(M.vertices.point_ptr(M.facets.vertex(f1,1)));
-        vec3 p3(M.vertices.point_ptr(M.facets.vertex(f1,2)));
-        vec3 q1(M.vertices.point_ptr(M.facets.vertex(f2,0)));
-        vec3 q2(M.vertices.point_ptr(M.facets.vertex(f2,1)));
-        vec3 q3(M.vertices.point_ptr(M.facets.vertex(f2,2)));
+        vec3 p1 = M.facets.point(f1,0);
+        vec3 p2 = M.facets.point(f1,1);
+        vec3 p3 = M.facets.point(f1,2);
+        vec3 q1 = M.facets.point(f2,0);
+        vec3 q2 = M.facets.point(f2,1);
+        vec3 q3 = M.facets.point(f2,2);
         return triangles_intersections(p1,p2,p3,q1,q2,q3,I);
     }
 
@@ -245,7 +220,7 @@ namespace GEO {
         delaunay_ = true;
         detect_intersecting_neighbors_ = true;
         use_radial_sort_ = true;
-        monster_threshold_ = index_t(-1);
+        monster_threshold_ = NO_INDEX;
         // TODO: understand why this breaks co-planarity tests,
         // with exact_nt it should have not changed anything !!
         // Anyway it does not seem to do any good, deactivated
@@ -321,6 +296,7 @@ namespace GEO {
     }
 
     void MeshSurfaceIntersection::intersect_prologue() {
+	Stopwatch W("Prologue", verbose_);
         if(!mesh_.facets.are_simplices()) {
             tessellate_facets(mesh_,3);
         }
@@ -364,12 +340,9 @@ namespace GEO {
                 );
             }
             double s = 1.0/normalize_radius_;
-            for(index_t v: mesh_.vertices) {
-                double* p = mesh_.vertices.point_ptr(v);
-                for(coord_index_t c=0; c<3; ++c) {
-                    p[c] = s * (p[c]-normalize_center_[c]);
-                }
-            }
+	    for(vec3& p: mesh_.vertices.points()) {
+		p = s * (p - normalize_center_);
+	    }
         }
 
         // Pre-scale everything by 2^20 to avoid underflows
@@ -388,37 +361,34 @@ namespace GEO {
     void MeshSurfaceIntersection::intersect_get_intersections(
         vector<IsectInfo>& intersections
     ) {
-        if(verbose_) {
-            Logger::out("Intersect") << "get intersections" << std::endl;
-        }
+	Stopwatch Wtot("Find isects", verbose_);
         {
-            Stopwatch W("Detect isect",verbose_);
+	    Stopwatch* W = new Stopwatch("AABB build", verbose_);
+            MeshFacetsAABB AABB(mesh_,AABB_INPLACE);
+	    delete W;
 
-            if(verbose_) {
-                Logger::out("Intersect") << "  build AABB" << std::endl;
-            }
-            MeshFacetsAABB AABB(mesh_,true);
             vector<std::pair<index_t, index_t> > FF;
 
             // Get candidate pairs of intersecting facets
-            if(verbose_) {
-                Logger::out("Intersect") << "  get candidate facet pairs"
-                                         << std::endl;
-            }
+	    W = new Stopwatch("AABB box-box", verbose_);
             AABB.compute_facet_bbox_intersections(
                 [&](index_t f1, index_t f2) {
                     // Needed (maybe I should change that in AABB class)
                     if(f1 == f2) {
                         return;
                     }
-                    geo_assert(f1 < f2);
+
+		    // No need to test f1 < f2, already done in
+		    // AABB::compute_facet_bbox_intersections()
+		    // (note: in indirect mode, it is i1 < i2,
+		    //  where f1 = element_in_leaf(i1) (resp. ... f2 ... i2)
 
                     // Optionally skip facet pairs that
                     // share a vertex or an edge
                     if(
                         !detect_intersecting_neighbors_ && (
-                            (mesh_.facets.find_adjacent(f1,f2)!=index_t(-1)) ||
-                            (mesh_.facets.find_common_vertex(f1,f2)!=index_t(-1))
+                            (mesh_.facets.find_adjacent(f1,f2)!=NO_INDEX) ||
+                            (mesh_.facets.find_common_vertex(f1,f2)!=NO_INDEX)
                         )
                     ) {
                         return;
@@ -426,12 +396,10 @@ namespace GEO {
                     FF.push_back(std::make_pair(f1,f2));
                 }
             );
+	    delete W;
 
             // Compute facet-facet intersections in parallel
-            if(verbose_) {
-                Logger::out("Intersect") << "  compute intersections"
-                                         << std::endl;
-            }
+	    W = new Stopwatch("AABB tri-tri", verbose_);
             Process::spinlock lock = GEOGRAM_SPINLOCK_INIT;
             parallel_for_slice(
                 0,FF.size(), [&](index_t b, index_t e) {
@@ -509,6 +477,7 @@ namespace GEO {
                     }
                 }
             );
+	    delete W;
         }
 
         // Apply a random permutation to the facets, so that
@@ -519,29 +488,26 @@ namespace GEO {
         // TODO: a version of parallel_for() with smarter
         // (dynamic) thread scheduling.
         {
+	    Stopwatch W("rnd_perm",verbose_);
             vector<index_t> reorder(mesh_.facets.nb());
             for(index_t f: mesh_.facets) {
                 reorder[f] = f;
             }
-
 	    GEO::random_shuffle(reorder.begin(),reorder.end());
-
-            for(IsectInfo& II: intersections) {
-                II.f1 = reorder[II.f1];
-                II.f2 = reorder[II.f2];
-            }
-            Permutation::invert(reorder);
-            mesh_.facets.permute_elements(reorder);
+	    for(IsectInfo& II: intersections) {
+		II.f1 = reorder[II.f1];
+		II.f2 = reorder[II.f2];
+	    }
+	    Permutation::invert(reorder);
+	    mesh_.facets.permute_elements(reorder);
         }
     }
 
     void MeshSurfaceIntersection::intersect_remesh_intersections(
         vector<IsectInfo>& intersections
     ) {
-        if(verbose_) {
-            Logger::out("Intersect") << "remesh intersections" << std::endl;
-        }
 
+	Stopwatch W("CDT",verbose_);
 
         // Keep track of original facet ids: they will be copied to the subfacets
         // whenever a facet is split.
@@ -564,8 +530,6 @@ namespace GEO {
         mesh_copy_.copy(mesh_);
 
         {
-            Stopwatch W("Remesh isect",verbose_);
-
             // Sort intersections by f1, so that all intersections between f1
             // and another facet appear as a contiguous sequence.
             GEO::sort(
@@ -608,14 +572,14 @@ namespace GEO {
                         index_t(start[i+1]-start[i])
                     );
                 }
-                Logger::out("Intersect") << "Intersections: "
-                                         << nb_intersections << std::endl;
-                Logger::out("Intersect") << "Intersected triangles: "
-                                         << nb_intersected_triangles
-                                         << std::endl;
-                Logger::out("Intersect") << "Max intersections in triangle: "
-                                         << max_intersections_in_triangle
-                                         << std::endl;
+                Logger::out("CDT") << "Intersections: "
+				   << nb_intersections << std::endl;
+                Logger::out("CDT") << "Intersected triangles: "
+				   << nb_intersected_triangles
+				   << std::endl;
+                Logger::out("CDT") << "Max intersections in triangle: "
+				   << max_intersections_in_triangle
+				   << std::endl;
             }
 
 
@@ -692,16 +656,17 @@ namespace GEO {
                         // of intersections).
                         if(e-b >= monster_threshold_) {
                             index_t f = intersections[b].f1;
-                            MIT.save("triangulation_"+String::to_string(f)+".geogram");
+                            MIT.save(
+				"triangulation_"+String::to_string(f)+".geogram"
+			    );
                             //MIT.save_constraints(
                             //    "constraints_"+String::to_string(f)+".geogram"
                             //);
-                            std::ofstream out("facet_"+String::to_string(f)+".obj");
-                            for(
-                                index_t lv=0; lv<mesh_copy_.facets.nb_vertices(f); ++lv
-                            ) {
-                                index_t v = mesh_copy_.facets.vertex(f,lv);
-                                vec3 p(mesh_copy_.vertices.point_ptr(v));
+                            std::ofstream out(
+				"facet_"+String::to_string(f)+".obj"
+			    );
+			    for(index_t v: mesh_copy_.facets.vertices(f)) {
+                                vec3 p = mesh_copy_.vertices.point(v);
                                 if(rescale_) {
                                     p=INV_SCALING*p;
                                 }
@@ -712,7 +677,8 @@ namespace GEO {
                             }
                             out << "f ";
                             for(
-                                index_t lv=0; lv<mesh_copy_.facets.nb_vertices(f); ++lv
+                                index_t lv=0;
+				lv<mesh_copy_.facets.nb_vertices(f); ++lv
                             ) {
                                 out << lv+1 << " ";
                             }
@@ -723,8 +689,9 @@ namespace GEO {
                         MIT.clear();
                     }
                     if(fine_verbose_) {
-                        Logger::out("Isect") << String::format("[%2d] done",int(tid))
-                                             << std::endl;
+                        Logger::out("Isect")
+			    << String::format("[%2d] done",int(tid))
+			    << std::endl;
                     }
 #ifdef TRIANGULATE_IN_PARALLEL
                 });
@@ -735,15 +702,11 @@ namespace GEO {
     void MeshSurfaceIntersection::intersect_epilogue(
         const vector<IsectInfo>& intersections
     ) {
-
-        if(verbose_) {
-            Logger::out("Intersect") << "epilogue" << std::endl;
-        }
-
+	Stopwatch W_epilogue("Epilogue", verbose_);
         // Vertices coming from intersections may land exactly
         // on an existing vertex (see #111)
         {
-            Stopwatch("Colocate newv",verbose_);
+	    Stopwatch W("I on v", verbose_);
             vector<index_t> v2v(mesh_.vertices.nb());
             for(index_t v : mesh_.vertices) {
                 v2v[v] = v;
@@ -755,7 +718,7 @@ namespace GEO {
                     // coming from an intersection, check whether
                     // it also exists as an intersection
                     if(vertex_to_exact_point_[v] == nullptr) {
-                        const double* xyz = mesh_.vertices.point_ptr(v);
+                        const vec3& xyz = mesh_.vertices.point(v);
                         ExactPoint K(xyz[0], xyz[1], xyz[2], 1.0);
                         auto it = exact_point_to_vertex_.find(K);
                         if(it != exact_point_to_vertex_.end()) {
@@ -771,6 +734,7 @@ namespace GEO {
         }
 
         if(interpolate_attributes_) {
+	    Stopwatch W("Attrib", verbose_);
             Attribute<index_t> original_facet_id(
                 mesh_.facets.attributes(), "original_facet_id"
             );
@@ -807,10 +771,10 @@ namespace GEO {
                     continue;
                 }
 
-                vec3 p1(mesh_.vertices.point_ptr(v1));
-                vec3 p2(mesh_.vertices.point_ptr(v2));
-                vec3 p3(mesh_.vertices.point_ptr(v3));
-                vec3 p(mesh_.vertices.point_ptr(v));
+                vec3 p1=mesh_.vertices.point(v1);
+                vec3 p2=mesh_.vertices.point(v2);
+                vec3 p3=mesh_.vertices.point(v3);
+                vec3 p =mesh_.vertices.point(v);
 
                 double a  = Geom::triangle_area(p1,p2,p3);
                 double a1 = Geom::triangle_area(p ,p2,p3);
@@ -884,28 +848,19 @@ namespace GEO {
         }
 
         if(normalize_) {
-            for(index_t v: mesh_.vertices) {
-                double* p = mesh_.vertices.point_ptr(v);
-                for(coord_index_t c=0; c<3; ++c) {
-                    p[c] = normalize_radius_*p[c] + normalize_center_[c];
-                }
-            }
+	    for(vec3& p: mesh_.vertices.points()) {
+		p = normalize_radius_ * p + normalize_center_;
+	    }
         }
-
     }
 
     void MeshSurfaceIntersection::intersect() {
-        if(verbose_) {
-            Logger::out("Intersect") << "start." << std::endl;
-        }
+	Stopwatch W("Intersect", verbose_);
         intersect_prologue();
         vector<IsectInfo> intersections;
         intersect_get_intersections(intersections);
         intersect_remesh_intersections(intersections);
         intersect_epilogue(intersections);
-        if(verbose_) {
-            Logger::out("Intersect") << "finished." << std::endl;
-        }
     }
 
     MeshSurfaceIntersection::ExactPoint MeshSurfaceIntersection::exact_vertex(
@@ -916,7 +871,7 @@ namespace GEO {
         if(p != nullptr) {
             return *p;
         }
-        const double* xyz = mesh_.vertices.point_ptr(v);
+        const vec3& xyz = mesh_.vertices.point(v);
         return ExactPoint(xyz[0], xyz[1], xyz[2], 1.0);
     }
 
@@ -972,7 +927,7 @@ namespace GEO {
         std::map<ExactPoint,index_t,ExactPointCompare>::iterator it;
         bool inserted;
         std::tie(it, inserted) = exact_point_to_vertex_.insert(
-            std::make_pair(p,index_t(-1))
+            std::make_pair(p,NO_INDEX)
         );
         if(!inserted) {
             return it->second;
@@ -1180,10 +1135,10 @@ namespace GEO {
         // Get connected components by traversing both alpha2 and alpha3 links,
         // and orient facets coherently
         index_t nb_components = 0;
-        vector<index_t> component(mesh_.facets.nb(), index_t(-1));
+        vector<index_t> component(mesh_.facets.nb(), NO_INDEX);
         {
             for(index_t f:mesh_.facets) {
-                if(component[f] == index_t(-1)) {
+                if(component[f] == NO_INDEX) {
                     std::stack<index_t> S;
                     component[f] = nb_components;
                     S.push(f);
@@ -1193,7 +1148,7 @@ namespace GEO {
 
                         {
                             index_t f2 = halfedges_.facet_alpha3(f1);
-                            if(component[f2] == index_t(-1)) {
+                            if(component[f2] == NO_INDEX) {
                                 component[f2]=component[f1];
                                 S.push(f2);
                             }
@@ -1201,7 +1156,7 @@ namespace GEO {
 
                         for(index_t le1=0; le1<3; ++le1) {
                             index_t f2 = mesh_.facets.adjacent(f1,le1);
-                            if(f2!=index_t(-1) && component[f2]==index_t(-1)) {
+                            if(f2!=NO_INDEX && component[f2]==NO_INDEX) {
 #ifdef GEO_DEBUG
                                 index_t le2 = mesh_.facets.find_adjacent(f2,f1);
                                 geo_debug_assert(
@@ -1222,15 +1177,11 @@ namespace GEO {
         // Compute the volume enclosed by each chart
 
         vector<double> chart_volume(nb_charts,0.0);
-        vec3 p0(0.0, 0.0, 0.0);
         for(index_t f: mesh_.facets) {
-            index_t v1 = mesh_.facets.vertex(f,0);
-            index_t v2 = mesh_.facets.vertex(f,1);
-            index_t v3 = mesh_.facets.vertex(f,2);
-            vec3 p1(mesh_.vertices.point_ptr(v1));
-            vec3 p2(mesh_.vertices.point_ptr(v2));
-            vec3 p3(mesh_.vertices.point_ptr(v3));
-            chart_volume[chart[f]] += Geom::tetra_signed_volume(p0,p1,p2,p3);
+            vec3 p1 = mesh_.facets.point(f,0);
+            vec3 p2 = mesh_.facets.point(f,1);
+            vec3 p3 = mesh_.facets.point(f,2);
+            chart_volume[chart[f]] += dot(p1,cross(p2,p3)) / 6.0;
         }
 
         for(index_t c=0; c<chart_volume.size(); ++c) {
@@ -1242,7 +1193,7 @@ namespace GEO {
 
         vector<double> max_chart_volume_in_component(nb_components, 0.0);
         vector<index_t> chart_with_max_volume_in_component(
-            nb_components, index_t(-1)
+            nb_components, NO_INDEX
         );
 
         for(index_t f: mesh_.facets) {
@@ -1265,6 +1216,8 @@ namespace GEO {
 
     void MeshSurfaceIntersection::build_Weiler_model() {
 
+	Stopwatch W("Weiler",verbose_);
+
         Attribute<bool> corner_is_on_border(
             mesh_.facet_corners.attributes(), "is_on_border"
         );
@@ -1343,7 +1296,7 @@ namespace GEO {
         // orientation bordered by non-manifold radial edges
         index_t nb_charts = get_surface_connected_components(mesh_, "chart");
         if(verbose_) {
-            Logger::out("Intersect")
+            Logger::out("Weiler")
                 << "Found " << nb_charts << " charts" << std::endl;
         }
 
@@ -1378,7 +1331,7 @@ namespace GEO {
         {
             index_t nb_regions = get_surface_connected_components(mesh_,"chart");
             if(verbose_) {
-                Logger::out("Intersect")
+                Logger::out("Weiler")
                     << "Found " << nb_regions << " regions" << std::endl;
             }
         }
@@ -1483,7 +1436,7 @@ namespace GEO {
         }
 
         if(I_.verbose_) {
-            Logger::out("Intersect") << "Found " << nb() << " bundles"
+            Logger::out("Weiler") << "Found " << nb() << " bundles"
                                      << std::endl;
         }
     }
@@ -1550,8 +1503,8 @@ namespace GEO {
             polyline_start_.push_back(B_.size());
         }
         if(I_.verbose_) {
-            Logger::out("Intersect") << "Found " <<nb()<< " polylines"
-                                     << std::endl;
+            Logger::out("Weiler") << "Found " <<nb()<< " polylines"
+				  << std::endl;
         }
     }
 
@@ -1572,7 +1525,7 @@ namespace GEO {
                 index_t v = I_.radial_bundles_.vertex(bndl,lv);
                 if(v_id[v] == NO_INDEX) {
                     v_id[v] = skeleton.vertices.create_vertex(
-                        mesh_.vertices.point_ptr(v)
+                        mesh_.vertices.point(v)
                     );
                     if(I_.radial_bundles_.nb_bundles_around_vertex(v) != 2) {
                         v_selection[v] = true;
@@ -1744,11 +1697,11 @@ namespace {
      *  bit set in a 32 bits integer
      * \param[in] x the integer
      * \return the position of the leftmost bit
-     *  set, or index_t(-1) if the specified integer
+     *  set, or NO_INDEX if the specified integer
      *  is zero.
      */
     inline index_t leftmost_bit_set(index_t x) {
-        index_t result = index_t(-1);
+        index_t result = NO_INDEX;
         for(index_t i=0; i<32; ++i) {
             if((x&1) != 0) {
                 result = i;
@@ -1764,9 +1717,7 @@ namespace {
 namespace GEO {
 
     void MeshSurfaceIntersection::classify(const std::string& expr) {
-        if(verbose_) {
-            Logger::out("Weiler") << "Classifying facets" << std::endl;
-        }
+	Stopwatch W("Classify", verbose_);
 
         // Takes as input a Weiler model, with duplicated interfaces,
         // operand bit (that indices for each triangle the set of operands
@@ -1831,14 +1782,14 @@ namespace GEO {
         for(index_t f: mesh_.facets) {
             facet_component[f] = NO_INDEX;
         }
-        // vector<index_t> facet_component(mesh_.facets.nb(), index_t(-1));
+        // vector<index_t> facet_component(mesh_.facets.nb(), NO_INDEX);
 
 
         vector<index_t> component_vertex; // one vertex per component
         vector<index_t> component_inclusion_bits;
         {
             for(index_t f:mesh_.facets) {
-                if(facet_component[f] == index_t(-1)) {
+                if(facet_component[f] == NO_INDEX) {
 
                     component_vertex.push_back(mesh_.facets.vertex(f,0));
                     component_inclusion_bits.push_back(0);
@@ -1852,8 +1803,8 @@ namespace GEO {
 
                         {
                             index_t f2 = halfedges_.facet_alpha3(f1);
-                            geo_debug_assert(f2 != index_t(-1));
-                            if(facet_component[f2] == index_t(-1)) {
+                            geo_debug_assert(f2 != NO_INDEX);
+                            if(facet_component[f2] == NO_INDEX) {
                                 facet_component[f2]=facet_component[f1];
                                 S.push(f2);
                             }
@@ -1862,8 +1813,8 @@ namespace GEO {
                         for(index_t le1=0; le1<3; ++le1) {
                             index_t f2 = mesh_.facets.adjacent(f1,le1);
                             if(
-                                f2 != index_t(-1) &&
-                                facet_component[f2] == index_t(-1)
+                                f2 != NO_INDEX &&
+                                facet_component[f2] == NO_INDEX
                             ) {
 #ifdef GEO_DEBUG
                                 index_t le2 = mesh_.facets.find_adjacent(f2,f1);
@@ -1884,15 +1835,11 @@ namespace GEO {
 
         // Compute the volume enclosed by each chart
         vector<double> chart_volume(nb_charts,0.0);
-        vec3 p0(0.0, 0.0, 0.0);
         for(index_t f: mesh_.facets) {
-            index_t v1 = mesh_.facets.vertex(f,0);
-            index_t v2 = mesh_.facets.vertex(f,1);
-            index_t v3 = mesh_.facets.vertex(f,2);
-            vec3 p1(mesh_.vertices.point_ptr(v1));
-            vec3 p2(mesh_.vertices.point_ptr(v2));
-            vec3 p3(mesh_.vertices.point_ptr(v3));
-            chart_volume[chart[f]] += Geom::tetra_signed_volume(p0,p1,p2,p3);
+            vec3 p1 = mesh_.facets.point(f,0);
+            vec3 p2 = mesh_.facets.point(f,1);
+            vec3 p3 = mesh_.facets.point(f,2);
+            chart_volume[chart[f]] += dot(p1,cross(p2,p3)) / 6.0;
         }
 
         // For each component, find the chart that encloses the largest
@@ -1900,7 +1847,7 @@ namespace GEO {
 
         vector<double> max_chart_volume_in_component(nb_components, 0.0);
         vector<index_t> chart_with_max_volume_in_component(
-            nb_components, index_t(-1)
+            nb_components, NO_INDEX
         );
 
         for(index_t f: mesh_.facets) {
@@ -2005,7 +1952,7 @@ namespace GEO {
                 S.pop();
                 {
                     index_t f2 = halfedges_.facet_alpha3(f1);
-                    if(f2 != index_t(-1) && !visited[f2]) {
+                    if(f2 != NO_INDEX && !visited[f2]) {
                         visited[f2] = true;
                         S.push(f2);
                         operand_inclusion_bits[f2] =
@@ -2014,7 +1961,7 @@ namespace GEO {
                 }
                 for(index_t le=0; le<3; ++le) {
                     index_t f2 = mesh_.facets.adjacent(f1,le);
-                    if(f2 != index_t(-1) && !visited[f2]) {
+                    if(f2 != NO_INDEX && !visited[f2]) {
                         visited[f2] = true;
                         S.push(f2);
                         operand_inclusion_bits[f2] = operand_inclusion_bits[f1];
@@ -2183,7 +2130,7 @@ namespace GEO {
         // v is an original vertex, we can use double-precision coords
         // everywhere.
 
-        vec3 p1(mesh_.vertices.point_ptr(v));
+        vec3 p1 = mesh_.vertices.point(v);
         vec3 p2 = p1 + vec3(
             1.0e6*(2.0*Numeric::random_float64()-1.0),
             1.0e6*(2.0*Numeric::random_float64()-1.0),
@@ -2207,15 +2154,9 @@ namespace GEO {
 
             bool degenerate = false;
 
-            vec3 q1(
-                mesh_copy_.vertices.point_ptr(mesh_copy_.facets.vertex(t,0))
-            );
-            vec3 q2(
-                mesh_copy_.vertices.point_ptr(mesh_copy_.facets.vertex(t,1))
-            );
-            vec3 q3(
-                mesh_copy_.vertices.point_ptr(mesh_copy_.facets.vertex(t,2))
-            );
+            vec3 q1 = mesh_copy_.facets.point(t,0);
+            vec3 q2 = mesh_copy_.facets.point(t,1);
+            vec3 q3 = mesh_copy_.facets.point(t,2);
 
             if(segment_triangle_intersection(p1,p2,q1,q2,q3,degenerate)) {
                 // If there was an intersection, change the parity
@@ -2308,16 +2249,13 @@ namespace GEO {
             return;
         }
 
-        if(verbose_) {
-            Logger::out("Intersect") << "Simplifying coplanar facets"
-                                     << std::endl;
-        }
+	Stopwatch W("Coplanar",verbose_);
         Attribute<index_t> facet_group(mesh_.facets.attributes(), "group");
         vector<index_t> group_facet; // one facet per group
         for(index_t f: mesh_.facets) {
-            facet_group[f] = index_t(-1);
+            facet_group[f] = NO_INDEX;
         }
-        Attribute<bool> keep_vertex(mesh_.facets.attributes(), "keep");
+        Attribute<bool> keep_vertex(mesh_.vertices.attributes(), "keep");
         for(index_t v: mesh_.vertices) {
             keep_vertex[v] = false;
         }
@@ -2414,11 +2352,7 @@ namespace {
         );
         index_t v_ofs = result.vertices.create_vertices(operand.vertices.nb());
         for(index_t v: operand.vertices) {
-            const double* p_src = operand.vertices.point_ptr(v);
-            double* p_dst = result.vertices.point_ptr(v + v_ofs);
-            p_dst[0] = p_src[0];
-            p_dst[1] = p_src[1];
-            p_dst[2] = p_src[2];
+	    result.vertices.point(v + v_ofs) = operand.vertices.point(v);
         }
         for(index_t f1: operand.facets) {
             index_t N = operand.facets.nb_vertices(f1);
@@ -2438,7 +2372,7 @@ namespace {
 namespace GEO {
 
     void mesh_boolean_operation(
-        Mesh& result, Mesh& A, Mesh& B,
+        Mesh& result, const Mesh& A, const Mesh& B,
         const std::string& operation,
 	MeshBooleanOperationFlags flags
     ) {
@@ -2490,21 +2424,13 @@ namespace GEO {
     }
 
     bool mesh_facets_have_intersection(Mesh& M, index_t f1, index_t f2) {
-        index_t cb1 = M.facets.corners_begin(f1);
-        index_t cb2 = M.facets.corners_begin(f2);
-        vec3 p0(M.vertices.point_ptr(M.facet_corners.vertex(cb1)));
-        vec3 q0(M.vertices.point_ptr(M.facet_corners.vertex(cb2)));
-        for(index_t c1 = cb1+1; c1+1<M.facets.corners_end(f1); ++c1) {
-            vec3 p1(M.vertices.point_ptr(M.facet_corners.vertex(c1)));
-            vec3 p2(M.vertices.point_ptr(M.facet_corners.vertex(c1+1)));
-            for(index_t c2 = cb2+1; c2+1<M.facets.corners_end(f2); ++c2) {
-                vec3 q1(M.vertices.point_ptr(M.facet_corners.vertex(c2)));
-                vec3 q2(M.vertices.point_ptr(M.facet_corners.vertex(c2+1)));
-                if(triangles_intersections(p0,p1,p2,q0,q1,q2)) {
-                    return true;
+	for(auto [ p1, p2, p3] : M.facets.triangle_points(f1)) {
+	    for(auto [ q1, q2, q3] : M.facets.triangle_points(f2)) {
+		if(triangles_intersections(p1,p2,p3,q1,q2,q3)) {
+		    return true;
                 }
-            }
-        }
-        return false;
+	    }
+	}
+	return false;
     }
 }
