@@ -176,8 +176,21 @@ namespace GEO {
      *  Default is unset.
      */
     void set_verbose(bool x) {
-        verbose_      = x;
+        verbose_ = x;
+	if(!verbose_ && fine_verbose_) {
+	    fine_verbose_ = false;
+	}
+    }
+
+    /**
+     * \brief Display detailed information while computing the intersection.
+     *  Default is unset.
+     */
+    void set_fine_verbose(bool x) {
         fine_verbose_ = x;
+	if(fine_verbose_ && !verbose_) {
+	    verbose_ = true;
+	}
     }
 
     /**
@@ -436,7 +449,7 @@ namespace GEO {
         RadialSort(
             const MeshSurfaceIntersection& mesh
         ) : mesh_(mesh),
-            h_ref_(index_t(-1)),
+            h_ref_(NO_INDEX),
             degenerate_(false)
         {
         }
@@ -1145,7 +1158,7 @@ namespace GEO {
      *   - MESH_BOOL_OPS_NO_SIMPLIFY: do not simplify coplanar facets
      */
     void GEOGRAM_API mesh_boolean_operation(
-        Mesh& result, Mesh& A, Mesh& B, const std::string& operation,
+        Mesh& result, const Mesh& A, const Mesh& B, const std::string& operation,
         MeshBooleanOperationFlags flags = MESH_BOOL_OPS_DEFAULT
     );
 
@@ -1160,7 +1173,7 @@ namespace GEO {
      *   during computation
      */
     inline void mesh_boolean_operation(
-        Mesh& result, Mesh& A, Mesh& B, const std::string& operation,
+        Mesh& result, const Mesh& A, const Mesh& B, const std::string& operation,
         bool verbose
     ) {
 	mesh_boolean_operation(
@@ -1182,7 +1195,7 @@ namespace GEO {
      *   - MESH_BOOL_OPS_NO_SIMPLIFY: do not simplify coplanar facets
      */
     inline void mesh_union(
-        Mesh& result, Mesh& A, Mesh& B,
+        Mesh& result, const Mesh& A, const Mesh& B,
 	MeshBooleanOperationFlags flags=MESH_BOOL_OPS_DEFAULT
     ) {
         mesh_boolean_operation(result, A, B, "A+B", flags);
@@ -1198,7 +1211,7 @@ namespace GEO {
      *  information during computation
      */
     inline void mesh_union(
-        Mesh& result, Mesh& A, Mesh& B, bool verbose
+        Mesh& result, const Mesh& A, const Mesh& B, bool verbose
     ) {
         mesh_boolean_operation(result, A, B, "A+B", verbose);
     }
@@ -1217,7 +1230,7 @@ namespace GEO {
      *   - MESH_BOOL_OPS_NO_SIMPLIFY: do not simplify coplanar facets
      */
     inline void mesh_intersection(
-        Mesh& result, Mesh& A, Mesh& B,
+        Mesh& result, const Mesh& A, const Mesh& B,
 	MeshBooleanOperationFlags flags=MESH_BOOL_OPS_DEFAULT
     ) {
         mesh_boolean_operation(result, A, B, "A*B", flags);
@@ -1233,7 +1246,7 @@ namespace GEO {
      *  during computation
      */
     inline void mesh_intersection(
-        Mesh& result, Mesh& A, Mesh& B, bool verbose
+        Mesh& result, const Mesh& A, const Mesh& B, bool verbose
     ) {
         mesh_boolean_operation(result, A, B, "A*B", verbose);
     }
@@ -1251,7 +1264,7 @@ namespace GEO {
      *   - MESH_BOOL_OPS_NO_SIMPLIFY: do not simplify coplanar facets
      */
     inline void mesh_difference(
-        Mesh& result, Mesh& A, Mesh& B,
+        Mesh& result, const Mesh& A, const Mesh& B,
 	MeshBooleanOperationFlags flags=MESH_BOOL_OPS_DEFAULT
     ) {
         mesh_boolean_operation(result, A, B, "A-B", flags);
@@ -1267,7 +1280,7 @@ namespace GEO {
      *  during computation
      */
     inline void mesh_difference(
-        Mesh& result, Mesh& A, Mesh& B, bool verbose
+        Mesh& result, const Mesh& A, const Mesh& B, bool verbose
     ) {
         mesh_boolean_operation(result, A, B, "A-B", verbose);
     }
