@@ -224,7 +224,10 @@ namespace {
         }
         std::string primitive_str(p1, size_t(p2-p1));
         for(int i=0; i<GLUP_NB_PRIMITIVES; ++i) {
-            if(primitive_str == GLUP::Context::glup_primitive_name(GLUPprimitive(i))) {
+            if(
+		primitive_str ==
+		GLUP::Context::glup_primitive_name(GLUPprimitive(i))
+	    ) {
                 return GLUPprimitive(i);
             }
         }
@@ -264,6 +267,7 @@ GLUPuint glupCompileProgram(const char* source_in) {
             targets.push_back(GL_VERTEX_SHADER);
             sources.push_back("//import <GLUPES/vertex_shader.h>\n");
         } else if(
+            !strcmp(glupCurrentProfileName(),"GLUP140") ||
             !strcmp(glupCurrentProfileName(),"GLUP150") ||
             !strcmp(glupCurrentProfileName(),"GLUP440")
         ) {
@@ -275,7 +279,8 @@ GLUPuint glupCompileProgram(const char* source_in) {
     GLuint program = 0;
     try {
         for(index_t i=0; i<index_t(sources.size()); ++i) {
-            GLUPuint shader = glupCompileShader(targets[i], primitive, sources[i]);
+            GLUPuint shader =
+		glupCompileShader(targets[i], primitive, sources[i]);
             if(shader == 0) {
 #ifdef GEO_OS_EMSCRIPTEN
                 return 0;
@@ -431,7 +436,9 @@ GLUPcontext glupCreateContext() {
             GLUP_profile = "GLUP440";
         } else if(GLSL_version >= 1.5) {
             GLUP_profile = "GLUP150";
-        } else {
+        } else if(GLSL_version >= 1.4) {
+            GLUP_profile = "GLUP140";
+	} else {
             GLUP_profile = "GLUPES2";
         }
 
@@ -466,7 +473,24 @@ GLUPcontext glupCreateContext() {
             result->setup();
         } catch(...) {
             GEO::Logger::warn("GLUP")
-                << "Caught an exception in GLUP150, downgrading to GLUPES2"
+                << "Caught an exception in GLUP150, downgrading to GLUP140"
+                << std::endl;
+            downgrade_message();
+            GLUP_profile = "GLUP140";
+            delete result;
+            result = nullptr;
+        }
+    }
+#endif
+
+#ifdef GEO_GL_140
+    if(GLUP_profile == "GLUP140") {
+        try {
+            result = new GLUP::Context_GLSL140;
+            result->setup();
+        } catch(...) {
+            GEO::Logger::warn("GLUP")
+                << "Caught an exception in GLUP140, downgrading to GLUPES2"
                 << std::endl;
             downgrade_message();
             GLUP_profile = "GLUPES2";
